import pytest

from insights.core.exceptions import ParseException
from insights.parsers import kdump
from insights.tests import context_wrap

KDUMP_WITH_NORMAL_COMMENTS = """
# this is a comment

ssh kdumpuser@10.209.136.62
path /kdump/raw
core_collector makedumpfile -c --message-level 1 -d 31
""".strip()

KDUMP_WITH_INLINE_COMMENTS = """
nfs4 10.209.136.62:/kdumps
path /kdump/raw #some path stuff
core_collector makedumpfile -c --message-level 1 -d 31
""".strip()

KDUMP_WITH_EQUAL = """
nfs 10.209.136.62:/kdumps
path /kdump/raw #some path stuff
core_collector makedumpfile -c --message-level 1 -d 31
some_var "blah=3"
options bonding mode=active-backup miimon=100
""".strip()

KDUMP_WITH_BLACKLIST = """
path /var/crash
core_collector makedumpfile -c --message-level 1 -d 24
default shell
blacklist vxfs
blacklist vxportal
blacklist vxted
blacklist vxcafs
blacklist fdd
ignore_me
"""

KDUMP_WITH_NET = """
net user@raw.server.com
path /var/crash
""".strip()

KDUMP_MATCH_1 = """
net user@raw.server.com
path /var/crash
""".strip()


def test_with_normal_comments():
    context = context_wrap(KDUMP_WITH_NORMAL_COMMENTS)
    kd = kdump.KDumpConf(context)
    expected = "# this is a comment"
    assert expected == kd.comments[0]
    # Also test is_* properties
    assert not kd.is_nfs()
    assert kd.is_ssh()
    # Not a local disk then.
    assert not kd.using_local_disk


def test_with_inline_comments():
    context = context_wrap(KDUMP_WITH_INLINE_COMMENTS)
    kd = kdump.KDumpConf(context)
    expected = "path /kdump/raw #some path stuff"
    assert expected == kd.inline_comments[0]
    assert "/kdump/raw" == kd["path"]
    # Also test is_* properties
    assert kd.is_nfs()
    assert not kd.is_ssh()
    # Not a local disk then.
    assert not kd.using_local_disk


def test_with_equal():
    context = context_wrap(KDUMP_WITH_EQUAL)
    kd = kdump.KDumpConf(context)
    expected = '"blah=3"'
    assert expected == kd['some_var']
    assert 'options' in kd.data
    assert isinstance(kd.data['options'], dict)
    assert 'bonding' in kd.data['options']
    assert 'mode=active-backup miimon=100' == kd.data['options']['bonding']
    # Alternate accessor for options:
    assert kd.options('bonding') == 'mode=active-backup miimon=100'
    # Also test is_* properties
    assert kd.is_nfs()
    assert not kd.is_ssh()
    # Not a local disk then.
    assert not kd.using_local_disk


def test_get_hostname():
    context = context_wrap(KDUMP_WITH_EQUAL)
    kd = kdump.KDumpConf(context)
    assert '10.209.136.62' == kd.hostname

    context = context_wrap(KDUMP_MATCH_1)
    kd = kdump.KDumpConf(context)
    assert 'raw.server.com' == kd.hostname


def test_get_ip():
    context = context_wrap(KDUMP_WITH_EQUAL)
    kd = kdump.KDumpConf(context)
    assert '10.209.136.62' == kd.ip

    context = context_wrap(KDUMP_MATCH_1)
    kd = kdump.KDumpConf(context)
    assert kd.ip is None


def test_blacklist_repeated():
    context = context_wrap(KDUMP_WITH_BLACKLIST)
    kd = kdump.KDumpConf(context)
    assert 'blacklist' in kd.data
    assert kd.data['blacklist'] == ['vxfs', 'vxportal', 'vxted', 'vxcafs', 'fdd']
    # Also test is_* properties
    assert not kd.is_nfs()
    assert not kd.is_ssh()
    assert kd.using_local_disk


def test_net():
    context = context_wrap(KDUMP_WITH_NET)
    kd = kdump.KDumpConf(context)
    assert 'net' in kd.data
    assert 'path' in kd.data
    assert not kd.using_local_disk
    with pytest.raises(KeyError):
        assert kd[3]


KEXEC_CRASH_SIZE_1 = "134217728"
KEXEC_CRASH_SIZE_2 = "0"
KEXEC_CRASH_SIZE_BAD = ""


def test_kexec_crash_size():
    kcs = kdump.KexecCrashSize(context_wrap(KEXEC_CRASH_SIZE_1))
    assert kcs.size == 134217728
    kcs = kdump.KexecCrashSize(context_wrap(KEXEC_CRASH_SIZE_2))
    assert kcs.size == 0
    kcs = kdump.KexecCrashSize(context_wrap(KEXEC_CRASH_SIZE_BAD))
    assert kcs.size == 0


KDUMP_CRASH_NOT_LOADED = '0'
KDUMP_CRASH_LOADED = '1'
KDUMP_CRASH_LOADED_BAD = ''


def test_loaded():
    ctx = context_wrap(KDUMP_CRASH_LOADED, path='/sys/kernel/kexec_crash_loaded')
    assert kdump.KexecCrashLoaded(ctx).is_loaded


def test_not_loaded():
    ctx = context_wrap(KDUMP_CRASH_NOT_LOADED, path='/sys/kernel/kexec_crash_loaded')
    assert not kdump.KexecCrashLoaded(ctx).is_loaded


def test_loaded_bad():
    ctx = context_wrap(KDUMP_CRASH_LOADED_BAD, path='/sys/kernel/kexec_crash_loaded')
    assert not kdump.KexecCrashLoaded(ctx).is_loaded


KDUMP_LOCAL_FS_1 = """
ext3 UUID=f15759be-89d4-46c4-9e1d-1b67e5b5da82
path /usr/local/cores
core_collector makedumpfile -c --message-level 1 -d 31
""".strip()

KDUMP_LOCAL_FS_UNSUPPORTED_2 = """
auto LABEL=/boot
path /usr/local/cores
core_collector makedumpfile -c --message-level 1 -d 31
""".strip()

KDUMP_REMOTE_TARGET_3 = """
net user@raw.server.com
path /usr/local/cores
core_collector makedumpfile -c --message-level 1 -d 31
""".strip()


def test_target():
    kd = kdump.KDumpConf(context_wrap(KDUMP_LOCAL_FS_1))
    assert kd.using_local_disk
    assert kd.target == ('ext3', 'UUID=f15759be-89d4-46c4-9e1d-1b67e5b5da82')
    assert kd['path'] == '/usr/local/cores'

    kd = kdump.KDumpConf(context_wrap(KDUMP_LOCAL_FS_UNSUPPORTED_2))
    assert kd.using_local_disk
    assert kd.target is None
    assert kd['path'] == '/usr/local/cores'

    kd = kdump.KDumpConf(context_wrap(KDUMP_REMOTE_TARGET_3))
    assert not kd.using_local_disk
    assert kd.target == ('net', 'user@raw.server.com')


KDUMP_TARGET_CONFLICT_1 = """
net user@raw.server.com
raw /dev/sda5
"""

KDUMP_TARGET_CONFLICT_2 = """
ext4 /dev/sdb1
ext4 UUID=f15759be-89d4-46c4-9e1d-1b67e5b5da82
"""


def test_conflict_targets_excptions():

    with pytest.raises(ParseException) as e_info:
        kdump.KDumpConf(context_wrap(KDUMP_TARGET_CONFLICT_1))
        assert "More than one target is configured" in str(e_info.value)

    with pytest.raises(ParseException) as e_info:
        kdump.KDumpConf(context_wrap(KDUMP_TARGET_CONFLICT_2))
        assert "More than one ext4 type targets" in str(e_info.value)



import pytest

from insights.parsers.sssd_conf import SSSDConf, SSSDConfd
from insights.tests import context_wrap

sssd_conf_cnt = """
[sssd]
config_file_version = 2
domains = example.com
debug_level = 0xfff0

[domain/example.com]
id_provider = ldap
enumerate = {0}
"""


@pytest.mark.parametrize("cls", [SSSDConf, SSSDConfd])
@pytest.mark.parametrize(
    "bool_val", ["True", "true", "TRUE", "False", "false", "FALSE"]
)
def test_sssd_conf(cls, bool_val):
    result = cls(
        context_wrap(sssd_conf_cnt.format(bool_val), path="/etc/sssd/sssd.conf")
    )

    assert result.file_name == "sssd.conf"
    assert result.file_path == "/etc/sssd/sssd.conf"

    assert "sssd" in result
    assert "domain/example.com" in result
    assert result.sections() == ["sssd", "domain/example.com"]

    assert result.getint("sssd", "config_file_version") == 2
    assert result.get("sssd", "domains") == "example.com"
    assert result.get("sssd", "debug_level") == "0xfff0"

    expected = bool_val.lower() == "true"
    assert result.get("domain/example.com", "id_provider") == "ldap"
    assert result.getboolean("domain/example.com", "enumerate") == expected



"""
crio configuration
==================
The crio files are normally available to rules as a list of CrioConf objects.
"""

from insights.core.exceptions import NoOptionError, NoSectionError
from insights.core.plugins import combiner
from insights.parsers.crio_conf import CrioConf


@combiner(CrioConf)
class AllCrioConf(object):
    """
    Combiner for accessing all the crio configuration files. There may be multi
    files for crio configuration, and the main config file is crio.conf. In the
    situation that the same section is both in crio.conf and other configuration
    files, the item in crio.conf has the lowest precedence. Files in the
    directory,'/etc/crio/crio.conf.d/', are sorted by name in lexical order and
    applied in that order. If multiple configuration files specify the same
    configuration option the setting specified in the file sorted last takes
    precedence over any other value. This combiner will parse all the CrioConf
    objects and return a dictionary containing all valid data.

    Sample files::

        /etc/crio/crio.conf:

            [crio]
            storage_driver = "overlay"
            storage_option = [
                    "overlay.override_kernel_check=1",
            ]

            [crio.runtime]
            selinux = true

            [crio.network]
            plugin_dirs = [
                    "/usr/libexec/cni",
            ]
            [crio.metrics]

        /etc/crio/crio.conf.d/00-conmon.conf

            [crio]
            internal_wipe = true
            storage_driver = "device mapper"

        /etc/crio/crio.conf.d/99-conmon.conf

            [crio]
            storage_driver = "overlay2"

            [crio.api]
            stream_address = ""
            stream_port = "10010"

            [crio.runtime]
            selinux = true
            conmon = ""
            conmon_cgroup = "pod"
            default_env = [
                "NSS_SDB_USE_CACHE=no",
            ]
            log_level = "info"
            cgroup_manager = "systemd"

    Examples:
        >>> all_crio_conf.sections()
        ['crio', 'crio.runtime', 'crio.api', 'crio.network', 'crio.metrics']
        >>> all_crio_conf.options('crio.api')
        ['stream_address', 'stream_port']
        >>> all_crio_conf.files
        ['/etc/crio/crio.conf', '/etc/crio/crio.conf.d/00-conmon.conf',
         '/etc/crio/crio.conf.d/99-conmon.conf']
        >>> all_crio_conf.get('crio', 'storage_driver')
        '"overlay2"'

    Attributes:
        files (list): The list of configuration file names.
    """

    def __init__(self, crio_confs):
        self.data = {}
        self.files = []
        conf_d_data = []

        def dict_merge(dest, src):
            if not src:
                return

            for section in src.sections():
                if section not in dest:
                    dest[section] = {}

                for option in src.items(section):
                    dest[section][option] = src.get(section, option)

        for crio_conf in crio_confs:
            self.files.append(crio_conf.file_path)
            if crio_conf.file_path == "/etc/crio/crio.conf":
                dict_merge(self.data, crio_conf)
            else:
                conf_d_data.append(crio_conf)

        conf_d_data.sort(key=lambda e: e.file_name)
        for crio_conf in conf_d_data:
            dict_merge(self.data, crio_conf)

        super(AllCrioConf, self).__init__()

    def get(self, section, option):
        """
        Args:
            section (str): The section str to search for.
            option (str): The option str to search for.

        Returns:
            str: Returns the value of the option in the specified section.
        """
        if section not in self.data:
            raise NoSectionError(section)

        header = self.data.get(section)
        if option not in header:
            raise NoOptionError(section, option)

        return header.get(option)

    def sections(self):
        """
        Return a list of section names.
        """
        return list(self.data.keys())

    def has_section(self, section):
        """
        Indicate whether the named section is present in the configuration.
        Return True if the given section is present, and False if not present.
        """
        return section in self.data

    def options(self, section):
        """
        Return a list of option names for the given section name.
        """
        return list(self.data[section].keys()) if self.has_section(section) else []

    def has_option(self, section, option):
        """
        Check for the existence of a given option in a given section.
        Return True if the given option is present, and False if not present.
        """
        if section not in self.data:
            return False
        return option in self.data[section]



import textwrap

import pytest

from insights.combiners.ipa import IPA
from insights.combiners.sssd_conf import SSSDConfAll
from insights.core.exceptions import SkipComponent
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.ipa_conf import IPAConfig
from insights.parsers.sssd_conf import SSSDConf
from insights.tests import context_wrap


@pytest.mark.parametrize("sssd_domain", ["ipa.test", "IPA"])
def test_ipa__server(sssd_domain):
    ipa_conf = textwrap.dedent(
        """
        [global]
        host = server.ipa.test
        basedn = dc=ipahcc,dc=test
        realm = ipa.test
        domain = ipa.test
        xmlrpc_uri = https://server.ipa.test/ipa/xml
        ldap_uri = ldapi://%2Frun%2Fslapd-IPAHCC-TEST.socket
        mode = production
        enable_ra = True
        ra_plugin = dogtag
        dogtag_version = 10
        """
    ).strip()

    sssd_conf = textwrap.dedent(
        """
        [sssd]
        domains = {domain}

        [domain/{domain}]
        id_provider = ipa
        ipa_server_mode = True
        """.format(
            domain=sssd_domain
        )
    ).strip()

    rpms = InstalledRpms(context_wrap("sssd-2.9.1-1.el9.x86_64"))
    ipa_conf = IPAConfig(context_wrap(ipa_conf))
    sssd_conf = SSSDConfAll(
        SSSDConf(context_wrap(sssd_conf, "/etc/sssd/sssd.conf")), []
    )

    ipa = IPA(ipa_conf, sssd_conf, rpms)

    assert ipa.sssd_ipa_domains == [sssd_domain]
    assert ipa.is_server
    assert ipa.is_client


@pytest.mark.parametrize("sssd_domain", ["ipa.test", "IPA"])
@pytest.mark.parametrize(
    "explicit_server_mode", [True, False], ids=["server_mode_set", "server_mode_unset"]
)
@pytest.mark.parametrize(
    "multidomain", [True, False], ids=["multidomain", "singledomain"]
)
def test_ipa__client(sssd_domain, explicit_server_mode, multidomain):
    ipa_conf = textwrap.dedent(
        """
        [global]
        basedn = dc=ipahcc,dc=test
        realm = ipa.test
        domain = ipa.test
        server = server.ipa.test
        host = client91.ipa.test
        xmlrpc_uri = https://server.ipa.test/ipa/xml
        enable_ra = True
        """
    ).strip()

    sssd_conf = textwrap.dedent(
        """
        [sssd]
        domains = {domain}

        [domain/{domain}]
        id_provider = ipa
        """.format(
            domain=sssd_domain
        )
    ).strip()

    if explicit_server_mode:
        sssd_conf += "\nipa_server_mode = false"

    if multidomain:
        sssd_conf += "\n\n[domain/ad]\nid_provider=ad"

    rpms = InstalledRpms(context_wrap("sssd-2.9.1-1.el9.x86_64"))
    ipa_conf = IPAConfig(context_wrap(ipa_conf))
    sssd_conf = SSSDConfAll(
        SSSDConf(context_wrap(sssd_conf, "/etc/sssd/sssd.conf")), []
    )

    ipa = IPA(ipa_conf, sssd_conf, rpms)

    assert ipa.sssd_ipa_domains == [sssd_domain]
    assert not ipa.is_server
    assert ipa.is_client


@pytest.mark.parametrize("sssd_domain", ["ipa.test", "IPA"])
def test_ipa__raises_skip(sssd_domain):
    ipa_conf = textwrap.dedent(
        """
        [global]
        basedn = dc=ipahcc,dc=test
        realm = ipa.test
        domain = ipa.test
        server = server.ipa.test
        host = client91.ipa.test
        xmlrpc_uri = https://server.ipa.test/ipa/xml
        enable_ra = True
        """
    ).strip()

    sssd_conf = textwrap.dedent(
        """
        [sssd]
        domains = {domain}

        [domain/{domain}]
        id_provider = ipa
        """.format(
            domain=sssd_domain
        )
    ).strip()

    rpms = InstalledRpms(context_wrap("other-package-2.9.1-1.el9.x86_64"))
    ipa_conf = IPAConfig(context_wrap(ipa_conf))
    sssd_conf = SSSDConfAll(
        SSSDConf(context_wrap(sssd_conf, "/etc/sssd/sssd.conf")), []
    )

    with pytest.raises(SkipComponent):
        IPA(ipa_conf, sssd_conf, rpms)



"""
Module to interact with Satellite Based Certificates
"""
from __future__ import print_function
import os
import logging

logger = logging.getLogger(__name__)

# RHSM and Subscription Manager
RHSM_CONFIG = None
try:
    from rhsm.config import initConfig
    from rhsm.certificate import create_from_pem
    RHSM_CONFIG = initConfig()
except ImportError:
    logger.debug("Could not load RHSM modules")


# Class for dealing with rhsmCertificates
class rhsmCertificate:

    try:
        PATH = RHSM_CONFIG.get('rhsm', 'consumerCertDir')
    except:
        PATH = "/etc/pki/consumer/"

    KEY = 'key.pem'
    CERT = 'cert.pem'

    @classmethod
    def keypath(cls):
        return os.path.join(cls.PATH, cls.KEY)

    @classmethod
    def certpath(cls):
        return os.path.join(cls.PATH, cls.CERT)

    @classmethod
    def read(cls):
        f = open(cls.keypath())
        key = f.read()
        f.close()
        f = open(cls.certpath())
        cert = f.read()
        f.close()
        return rhsmCertificate(key, cert)

    @classmethod
    def exists(cls):
        return (os.path.exists(cls.keypath()) and
                os.path.exists(cls.certpath()))

    @classmethod
    def existsAndValid(cls):
        if cls.exists():
            try:
                cls.read()
                return True
            except Exception as e:
                print(e)
        return False

    def __init__(self, keystring, certstring):
        self.key = keystring
        self.cert = certstring
        self.x509 = create_from_pem(certstring)

    def getConsumerId(self):
        subject = self.x509.subject
        return subject.get('CN')

    def getConsumerName(self):
        altName = self.x509.alt_name
        return altName.replace("DirName:/CN=", "")

    def getSerialNumber(self):
        return self.x509.serial

    def __str__(self):
        return 'consumer: name="%s", uuid%s' % \
               (self.getConsumerName(),
                self.getConsumerId())



from insights.parsers import krb5
from insights.tests import context_wrap

CONF_CONTENT = """
[plugins]
localauth = {
   module = sssd:/usr/lib64/sssd/modules/sssd_krb5_localauth_plugin.so
 }
""".strip()


def test_krb5_localauth_plugin_conf():
    result = krb5.Krb5LocalauthPlugin(context_wrap(CONF_CONTENT))
    assert (
        result['plugins']['localauth']['module']
        == 'sssd:/usr/lib64/sssd/modules/sssd_krb5_localauth_plugin.so'
    )



import pytest

from insights import SkipComponent
from insights.components.selinux import SELinuxEnabled
from insights.parsers.selinux_config import SelinuxConfig
from insights.tests import context_wrap

SELINUX_CONFIG = """
SELINUX=enforcing
SELINUXTYPE=targeted
"""
SELINUX_CONFIG_DISABLED = """
SELINUX=disabled
SELINUXTYPE=targeted
"""


def test_selinux_enabled():
    conf = SelinuxConfig(context_wrap(SELINUX_CONFIG))
    result = SELinuxEnabled(conf)
    assert isinstance(result, SELinuxEnabled)

    conf = SelinuxConfig(context_wrap(SELINUX_CONFIG_DISABLED))
    with pytest.raises(SkipComponent):
        SELinuxEnabled(conf)



from insights.parsers.vmcore_dmesg import VMCoreDmesg
from insights.parsers import vmcore_dmesg
from insights.tests import context_wrap
import doctest


VMCORE_DMESG_TXT = """
[    0.000000]   A0000-BFFFF uncachable
[    0.000000]   C0000-DFFFF write-protect
[    0.000000]   E0000-FFFFF uncachable
[    0.000000] MTRR variable ranges enabled:
[    0.000000]   0 base 0000FE000000 mask 3FFFFF000000 uncachable
[    0.000000]   5 base 0000C0000000 mask 3FFFE0000000 uncachable
[    0.000000]   6 base 000080000000 mask 3FFFC0000000 uncachable
[    0.000000]   8 base 0000FF000000 mask 3FFFFF000000 uncachable
[    0.000000]   9 disabled
[    0.000000] PAT configuration [0-7]: WB  WC  UC- UC  WB  WP  UC- UC
[    0.000000] e820: last_pfn = 0x7b800 max_arch_pfn = 0x400000000
[    0.000000] Base memory trampoline at [ffff8d4380097000] 97000 size 24576
[    0.000000] Using GB pages for direct mapping
[    0.000000] BRK [0xabf22e000, 0xabf22efff] PGTABLE
[    0.000000] BRK [0xabf22f000, 0xabf22ffff] PGTABLE
[    0.000000] BRK [0xabf232000, 0xabf232fff] PGTABLE
""".strip()


VMCORE_DMESG_CRASH = """
[  345.691798] device-mapper: raid: Failed to read superblock of device at position 0
[  345.693497] device-mapper: raid: Discovered old metadata format; upgrading to extended metadata format
[  345.701166] md/raid1:mdX: active with 1 out of 2 mirrors
[  345.726870] BUG: unable to handle kernel NULL pointer dereference at 00000000000005ec
[  345.727782] IP: [<ffffffffc0852ffb>] read_balance+0x1db/0x4e0 [raid1]
[  345.728570] PGD 0
[  345.729279] Oops: 0000 [#1] SMP
[  345.729950] Modules linked in: raid1 dm_raid raid456 async_raid6_recov async_memcpy async_pq raid6_pq async_xor xor async_tx
[  345.734752]  drm_kms_helper syscopyarea sysfillrect sysimgblt fb_sys_fops ttm drm bfa ahci crct10dif_pclmul crct10dif_common
[  345.737334] CPU: 6 PID: 952 Comm: systemd-udevd Kdump: loaded Not tainted 3.10.0-862.9.1.el7.x86_64 #1
[  345.738198] Hardware name: Dell Inc. PowerEdge R730xd/072T6D, BIOS 2.4.3 01/17/2017
[  345.739334] task: ffff9f888d030fd0 ti: ffff9f86b76b4000 task.ti: ffff9f86b76b4000
""".strip()


TOKEN_KEYS = ['wp', 'conf', 'pg']
INDICATORS = ['write-protect', 'PAT configuration', 'BRK']


for key, indicator in zip(TOKEN_KEYS, INDICATORS):
    VMCoreDmesg.token_scan(key, indicator)


def test_vmcore_dmesg_lines():
    vm_core = VMCoreDmesg(context_wrap(VMCORE_DMESG_TXT))
    assert vm_core.lines == VMCORE_DMESG_TXT.splitlines()


def test_vmcore_dmesg_tokens():
    vm_core = VMCoreDmesg(context_wrap(VMCORE_DMESG_TXT))
    for key in TOKEN_KEYS:
        assert getattr(vm_core, key)


def test_vmcore_dmesg_doctest():
    env = {
        'vmcore_dmesg': VMCoreDmesg(context_wrap(VMCORE_DMESG_CRASH)),
    }
    failed, total = doctest.testmod(vmcore_dmesg, globs=env)
    assert failed == 0



import mock
import pytest

from mock.mock import Mock
from collections import defaultdict

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.rpm import LocalSpecs, pkgs_with_writable_dirs, rpm_v_pkg_list
from insights.specs import Specs

RPM_CMD = """
httpd-core; httpd-core-2.4.53-7.el9.x86_64; /usr/share/doc/httpd-core; drwxr-xr-x; apache; root; Red Hat, Inc.
httpd-core; httpd-core-2.4.53-7.el9.x86_64; /usr/share/doc/httpd-core/CHANGES; -rw-r--r--; root; root; Red Hat, Inc.
postgresql-server; postgresql-server-13.7-1.el9_0.x86_64; /var/lib/pgsql; drwx------; postgres; postgres; Red Hat, Inc.
polkit; polkit-0.117-10.el9_0.x86_64; /usr/lib/polkit-1; drwxrwxr-x; polkitd; polkitd; Red Hat, Inc.
polkit; polkit-0.117-10.el9_0.x86_64; /usr/lib/polkit-1/polkitd; -rwxr-xr-x; root; root; Red Hat, Inc.
""".strip()

RPM_EXPECTED = ["httpd-core|httpd-core-2.4.53-7.el9.x86_64|Red Hat, Inc."]

RPM_BAD_CMD = "bash: rpm: command not found..."

RPM_EMPTY_CMD = ""

RELATIVE_PATH = "insights_datasources/rpm_pkgs"


def setup_function(func):
    if func is test_rpm_v_pkgs:
        filters.add_filter(
            Specs.rpm_V_package_list,
            [
                'coreutils',
                'procps',
                'procps-ng',
                'shadow-utils',
                'passwd',
                'sudo',
                'chrony',
                'findutils',
                'glibc',
            ],
        )


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(dict)


def get_users():
    return ["apache", "postgres"]


def get_groups(users):
    return ["apache", "postgres"]


@mock.patch("insights.specs.datasources.rpm.get_users", get_users)
@mock.patch("insights.specs.datasources.rpm.get_groups", get_groups)
def test_rpm():
    rpm_args = Mock()
    rpm_args.content = RPM_CMD.splitlines()
    broker = {LocalSpecs.rpm_args: rpm_args}

    result = pkgs_with_writable_dirs(broker)
    expected = DatasourceProvider(content=RPM_EXPECTED, relative_path=RELATIVE_PATH)
    assert result
    assert isinstance(result, DatasourceProvider)
    assert sorted(result.content) == sorted(expected.content)
    assert result.relative_path == expected.relative_path


@pytest.mark.parametrize("no_rpm", [RPM_BAD_CMD, RPM_EMPTY_CMD])
def test_no_rpm(no_rpm):
    rpm_args = Mock()
    rpm_args.content = no_rpm.splitlines()
    broker = {LocalSpecs.rpm_args: rpm_args}

    with pytest.raises(SkipComponent):
        pkgs_with_writable_dirs(broker)


def test_pkgs_list_empty():
    with pytest.raises(SkipComponent):
        rpm_v_pkg_list({})


def test_rpm_v_pkgs():
    ret = rpm_v_pkg_list({})
    assert ret == [
        'chrony',
        'coreutils',
        'findutils',
        'glibc',
        'passwd',
        'procps',
        'procps-ng',
        'shadow-utils',
        'sudo',
    ]



"""
Hostname Obfuscation
====================
"""

import logging
import hashlib
import os
import re
import six

from insights.cleaner.utilities import write_report

logger = logging.getLogger(__name__)


class Hostname(object):
    """
    Class for obfuscating hostname.

    .. note::

        Currently, only the system hostname will be obfuscated, see:
        - https://docs.redhat.com/en/documentation/red_hat_insights/1-latest/html/client_configuration_guide_for_red_hat_insights/assembly-client-data-obfuscation#proc-obfuscating-hostname_insights-cg-obfuscation

    """

    def __init__(self, fqdn):
        # - Hostname obfuscate information
        # - original hostname : obfuscated hostname
        self._hn_db = dict()
        fqdn_split = fqdn.split('.')
        self.pattern = (
            None
            if len(fqdn_split) <= 1
            else r'(?![\W\-\:\ \.])[a-zA-Z0-9\-\_\.]*\.{0}'.format('.'.join(fqdn_split[1:]))
        )
        self._hostname = fqdn_split[0]
        self._hn2db(fqdn)

    def _hn2db(self, hn):
        '''
        Add a hostname to DB or return an existing entry
        '''
        # Check db first
        if hn in self._hn_db:
            return self._hn_db[hn]
        # Handle domain
        hn_sp = hn.split('.')
        obf_hn = hashlib.sha1(hn_sp[0].encode('utf-8') if six.PY3 else hn_sp[0]).hexdigest()[:12]
        if len(hn_sp) > 1:
            obf_hn = '{0}.{1}'.format(obf_hn, 'example.com')
        self._hn_db[hn] = obf_hn
        return obf_hn

    def parse_line(self, line, **kwargs):
        '''
        Replace the exact hostname and all instances of the known domain with the obfuscated alternatives.
        '''
        if not line:
            return line
        try:
            if self.pattern:
                hostnames = [each for each in re.findall(self.pattern, line)]
                for hn in hostnames:
                    new_hn = self._hn2db(hn)
                    logger.debug("Obfuscating FQDN - {0} > {1}".format(hn, new_hn))
                    line = line.replace(hn, new_hn)
            # catch any non-fqdn instances of the system hostname
            if self._hostname in line:
                line = line.replace(self._hostname, self._hn2db(self._hostname))
            return line
        except Exception as e:  # pragma: no cover
            logger.warning(e)
            raise Exception('SubHostnameError: Unable to Substitute Hostname/Domainname')

    def mapping(self):
        mapping = []
        for ori, obf in self._hn_db.items():
            mapping.append({'original': ori, 'obfuscated': obf})
        return mapping

    def generate_report(self, report_dir, archive_name):
        try:
            hn_report_file = os.path.join(report_dir, "%s-hostname.csv" % archive_name)
            logger.info('Creating Hostname Report - %s', hn_report_file)
            lines = ['Obfuscated Hostname,Original Hostname']
            if self._hn_db:
                for ori, obf in self._hn_db.items():
                    lines.append('{0},{1}'.format(obf, ori))
            else:  # pragma: no cover # never false
                lines.append('None,None')
        except Exception as e:  # pragma: no cover
            logger.exception(e)
            raise Exception('CreateReport Error: Error Creating Hostname Report')

        write_report(lines, hn_report_file)

        logger.info('Completed Hostname Report.')



"""
SSSD Configuration
==================

Provides access to complete SSSD configuration: /etc/sssd/sssd.conf with merged
configuration snippets from /etc/sssd/conf.d.
"""

from copy import deepcopy

from insights.core.exceptions import SkipComponent
from insights.core.plugins import combiner
from insights.parsers.sssd_conf import SSSDConf, SSSDConfd


@combiner(optional=[SSSDConf, SSSDConfd])
class SSSDConfAll(object):
    """
    Provides access to complete SSSD configuration: /etc/sssd/sssd.conf with
    merged configuration snippets from /etc/sssd/conf.d.
    """

    def __init__(self, sssd_conf=None, sssd_conf_d=None):
        if sssd_conf is None and not sssd_conf_d:
            raise SkipComponent("SSSD is not configured")

        conf = sssd_conf
        conf_d = []

        if sssd_conf_d is not None:
            conf_d = sorted(sssd_conf_d, key=lambda x: x.file_name)

        if conf is None:
            conf = conf_d.pop(0)

        self.config = deepcopy(conf)

        for parser in conf_d:
            if parser.file_name.startswith("."):
                continue

            for section in parser.sections():
                for key, value in parser.items(section).items():
                    self.config._set(section, key, value)

        self._enabled_domains = None

    @property
    def enabled_domains(self):
        """
        Returns the list of enabled domains.

        Domains can be enabled either using the ``domains`` option in the
        ``sssd`` section of the configuration file or using the ``enabled``
        option in the domain configuration.

        [sssd]
        domains = a, b

        [domain/a]
        ...

        [domain/b]
        ...

        [domain/c]
        enabled = true

        """
        if self._enabled_domains is None:
            enabled_domains = []

            if self.config.has_option("sssd", "domains"):
                domains = self.config.get("sssd", "domains")
                enabled_domains = [domain.strip() for domain in domains.split(",")]

            prefix = "domain/"
            for section in self.config.sections():
                # Ignore if this is not a domain configuration
                if not section.startswith(prefix):
                    continue

                name = section[len(prefix):].strip()
                if not name:
                    # Invalid configuration
                    continue

                # Ignore if this is a subdomain configuration
                # `domain/$dom/$subdom`
                if "/" in name:
                    continue

                if self.config.has_option(section, "enabled"):
                    enabled = self.config.getboolean(section, "enabled")

                    if enabled and name not in enabled_domains:
                        enabled_domains.append(name)
                    elif not enabled and name in enabled_domains:
                        enabled_domains.remove(name)

            self._enabled_domains = enabled_domains

        return self._enabled_domains

    def domain_config(self, domain):
        """
        Return the configuration dictionary for a specific domain, given as
        the raw name as listed in the 'domains' property of the sssd section.
        This then looks for the equivalent 'domain/{domain}' section of the
        config file.
        """
        full_domain = self.domain_section(domain)
        if full_domain not in self.config:
            return {}

        return self.config.items(full_domain)

    def domain_section(self, domain):
        """
        Transform plain SSSD domain name into a configuration section.

        ipa.test -> domain/ipa.test

        Args:
            domain (str): SSSD domain name.

        Returns:
            str: Returns the configuration section.
        """
        return "domain/" + domain

    def domain_get(self, domain, option, default=None):
        """
        Lookup option in domain.

        Args:
            domain (str): The SSSD domain name.
            option (str): The option str to search for.
            default (any): Default value if the option is not found.

        Returns:
            str: Returns the value of the option in the specified section.
        """
        section = self.domain_section(domain)

        if not self.config.has_option(section, option):
            return default

        return self.config.get(section, option)

    def domain_getboolean(self, domain, option, default=None):
        """
        Lookup boolean option in domain.

        Args:
            domain (str): The SSSD domain name.
            option (str): The option str to search for.
            default (any): Default value if the option is not found.

        Returns:
            bool: Returns boolean form based on the data from get.
        """
        section = self.domain_section(domain)

        if not self.config.has_option(section, option):
            return default

        return self.config.getboolean(section, option)



"""
Custom datasource to get the current kernel version.
"""

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.parsers.lsmod import LsMod
from insights.parsers.uname import Uname
from insights.specs import Specs


@datasource(Uname, HostContext)
def current_version(broker):
    """
    This datasource provides the current booting kernel version.

    Sample data returned::

        '4.18.0-240.el8.x86_64'

    Returns:
        String: The current kernel version.

    Raises:
        UnameError: When there is a problem occurs with uname data.
    """

    return broker[Uname].kernel


@datasource(Specs.grubby_default_kernel, HostContext)
def default_version(broker):
    """
    This datasource provides the default kernel version.

    Sample data returned::

        '4.18.0-240.el8.x86_64'

    Returns:
        String: The default kernel version.

    Raises:
        SkipComponent: When output is empty or an error occurs
    """
    try:
        content = broker[Specs.grubby_default_kernel].content
        if len(content) == 1 and len(content[0].split()) == 1:
            default_kernel = content[0]
            return default_kernel.lstrip("/boot/vmlinuz-")
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))
    raise SkipComponent


@datasource(LsMod, HostContext)
def kernel_module_filters(broker):
    """
    Return a string of a list of modules from the spec filter,
    separated with space.
    """
    filters = sorted((get_filters(Specs.modinfo_modules)))
    if filters:
        loaded_modules = []
        for item in filters:
            module_list = [module for module in broker[LsMod].data if item in module]
            if module_list:
                loaded_modules.extend(module_list)
        if loaded_modules:
            return str(' '.join(loaded_modules))
        raise SkipComponent
    raise SkipComponent



"""
SatelliteYaml - file ``/etc/foreman-installer/scenarios.d/satellite.yaml``
==========================================================================

Parse the file ``/etc/foreman-installer/scenarios.d/satellite.yaml``.

Sample input::

    ---
    :answer_file: /etc/foreman-installer/scenarios.d/satellite-answers.yaml
    :installer_dir: /usr/share/foreman-installer/katello
    :custom:
        :lock_package_versions: true
    :facts:
        tuning: default
        mongo_cache_size: 6.25

Examples:
    >>> ':facts' in SatelliteYaml
    True
    >>> SatelliteYaml[':facts']['tuning']
    'default'
"""

from insights.core.plugins import parser
from insights.core import YAMLParser
from insights.specs import Specs


@parser(Specs.satellite_yaml)
class SatelliteYaml(YAMLParser):
    pass



import doctest
from insights.parsers import login_pam_conf
from insights.parsers.login_pam_conf import LoginPamConf
from insights.tests import context_wrap

LOGIN_PAM_CONF = """
#%PAM-1.0
auth [user_unknown=ignore success=ok ignore=ignore default=bad] pam_securetty.so
auth       substack     system-auth
auth       include      postlogin
account    required     pam_nologin.so
account    include      system-auth
password   include      system-auth
# pam_selinux.so close should be the first session rule
session    required     pam_selinux.so close
session    required     pam_loginuid.so
session    optional     pam_console.so
""".strip()


def test_login_pam_conf():
    pam_conf = LoginPamConf(context_wrap(LOGIN_PAM_CONF, path='etc/pamd.d/login'))
    assert len(pam_conf) == 9
    assert pam_conf[0].service == 'login'
    assert pam_conf[0].interface == 'auth'
    assert len(pam_conf[0].control_flags) == 4
    assert pam_conf[0].control_flags[0].flag == 'user_unknown'
    assert pam_conf[0].control_flags[0].value == 'ignore'
    assert pam_conf[0].module_name == 'pam_securetty.so'
    assert pam_conf[6].interface == 'session'
    assert pam_conf[6].module_args == 'close'


def test_doc():
    env = {
        "login_pam_conf": LoginPamConf(context_wrap(LOGIN_PAM_CONF)),
    }
    failed_count, total = doctest.testmod(login_pam_conf, globs=env)
    assert failed_count == 0



"""
KeystoneConf - file ``/etc/keystone/keystone.conf``
===================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.keystone_conf)
class KeystoneConf(IniConfigFile):
    """
    The ``KeystoneConf`` class parses the information in the file
    ``/etc/keystone/keystone.conf``.  See the ``IniConfigFile`` class for more
    information on attributes and methods.

    Sample input data looks like::

        [DEFAULT]
        admin_token = ADMIN
        compute_port = 8774

        [identity]
        default_domain_id = default
        domain_configurations_from_database = false

        [identity_mapping]
        driver = keystone.identity.mapping_backends.sql.Mapping
        generator = keystone.identity.id_generators.sha256.Generator

    Examples:
        >>> type(conf)
        <class 'insights.parsers.keystone.KeystoneConf'>
        >>> 'identity' in conf
        True
        >>> conf.has_option('identity', 'default_domain_id')
        True
        >>> conf.has_option('identity', 'domain_specific_drivers_enabled')
        False
        >>> conf.get('identity', 'default_domain_id')
        'default'
    """
    pass



# flake8: noqa
LVMCONFIG = """
  WARNING: Running as a non-root user. Functionality may be unavailable.
devices {
	filter=["a|.*/|"]
	allow_changes_with_duplicate_pvs=0
	issue_discards=0
	pv_min_size=2048
	require_restorefile_with_uuid=1
	disable_after_error_count=0
	ignore_lvm_mirrors=1
	ignore_suspended_devices=0
	data_alignment_offset_detection=1
	data_alignment=0
	data_alignment_detection=1
	md_chunk_alignment=1
	fw_raid_component_detection=0
	md_component_detection=1
	multipath_component_detection=1
	sysfs_scan=1
	write_cache_state=1
	cache_file_prefix=""
	cache_dir="/etc/lvm/cache"
	preferred_names=["^/dev/mpath/","^/dev/mapper/mpath","^/dev/[hs]d"]
	external_device_info_source="none"
	obtain_device_list_from_udev=1
	scan="/dev"
	dir="/dev"
	global_filter=["a|.*/|"]
	cache="/etc/lvm/cache/.cache"
	default_data_alignment=1
}
backup {
	backup=1
	backup_dir="/etc/lvm/backup"
	archive=1
	archive_dir="/etc/lvm/archive"
	retain_min=10
	retain_days=30
}
shell {
	history_size=100
}
config {
	checks=1
	abort_on_errors=0
	profile_dir="/etc/lvm/profile"
}
allocation {
	thin_pool_metadata_require_separate_pvs=0
	cache_pool_metadata_require_separate_pvs=0
	mirror_logs_require_separate_pvs=0
	wipe_signatures_when_zeroing_new_lvs=1
	use_blkid_wiping=1
	maximise_cling=1
	raid_stripe_all_devices=0
	cache_pool_cachemode="writethrough"
	cache_metadata_format=0
	cache_mode="writethrough"
	thin_pool_zero=1
	thin_pool_discards="passdown"
	thin_pool_chunk_size_policy="generic"
	physical_extent_size=4096
}
log {
	report_command_log=0
	debug_classes=["memory","devices","io","activation","allocation","lvmetad","metadata","cache","locking","lvmpolld","dbus"]
	activation=0
	prefix="  "
	command_names=0
	indent=1
	level=0
	overwrite=0
	syslog=1
	silent=0
	verbose=0
	command_log_sort="log_seq_num"
	command_log_cols="log_seq_num,log_type,log_context,log_object_type,log_object_name,log_object_id,log_object_group,log_object_group_id,log_message,log_errno,log_ret_code"
	command_log_selection="!(log_type=status && message=success)"
}
global {
	fallback_to_lvm1=0
	notify_dbus=1
	use_lvmpolld=1
	system_id_source="none"
	use_lvmlockd=0
	use_lvmetad=1
	sparse_segtype_default="thin"
	raid10_segtype_default="raid10"
	mirror_segtype_default="raid1"
	metadata_read_only=0
	abort_on_internal_errors=0
	prioritise_write_locks=1
	locking_dir="/run/lock/lvm"
	fallback_to_local_locking=1
	fallback_to_clustered_locking=1
	wait_for_locks=1
	locking_type=1
	etc="/etc"
	proc="/proc"
	activation=1
	suffix=1
	si_unit_consistency=1
	units="r"
	test=0
	umask=63
	format="lvm2"
	locking_library="liblvm2clusterlock.so"
	detect_internal_vg_cache_corruption=0
	lvdisplay_shows_full_device_path=0
	use_aio=1
	lvmetad_update_wait_time=10
	lvmlockd_lock_retries=3
	sanlock_lv_extend=256
	thin_check_executable="/usr/sbin/thin_check"
	thin_dump_executable="/usr/sbin/thin_dump"
	thin_repair_executable="/usr/sbin/thin_repair"
	thin_check_options=["-q","--clear-needs-check-flag"]
	thin_repair_options=[""]
	cache_check_executable="/usr/sbin/cache_check"
	cache_dump_executable="/usr/sbin/cache_dump"
	cache_repair_executable="/usr/sbin/cache_repair"
	cache_check_options=["-q","--clear-needs-check-flag"]
	cache_repair_options=[""]
	fsadm_executable="/usr/sbin/fsadm"
}
activation {
	activation_mode="degraded"
	polling_interval=15
	monitoring=1
	use_mlockall=0
	thin_pool_autoextend_percent=20
	thin_pool_autoextend_threshold=100
	snapshot_autoextend_percent=20
	snapshot_autoextend_threshold=100
	mirror_log_fault_policy="allocate"
	mirror_image_fault_policy="remove"
	raid_fault_policy="warn"
	readahead="auto"
	raid_region_size=2048
	process_priority=-18
	reserved_memory=8192
	reserved_stack=64
	use_linear_target=1
	missing_stripe_filler="error"
	retry_deactivation=1
	verify_udev_operations=0
	udev_rules=1
	udev_sync=1
	checks=0
	mirror_region_size=2048
	error_when_full=0
	mirror_device_fault_policy="remove"
	auto_set_activation_skip=1
}
metadata {
	check_pv_device_sizes=1
	record_lvs_history=0
	lvs_history_retention_time=0
	pvmetadatacopies=1
	vgmetadatacopies=0
	pvmetadatasize=255
	pvmetadataignore=0
	stripesize=64
}
report {
	output_format="basic"
	compact_output=0
	compact_output_cols=""
	aligned=1
	buffered=1
	headings=1
	separator=" "
	list_item_separator=","
	prefixes=0
	quoted=1
	columns_as_rows=0
	binary_values_as_numeric=0
	time_format="%Y-%m-%d %T %z"
	devtypes_sort="devtype_name"
	devtypes_cols="devtype_name,devtype_max_partitions,devtype_description"
	devtypes_cols_verbose="devtype_name,devtype_max_partitions,devtype_description"
	lvs_sort="vg_name,lv_name"
	lvs_cols="lv_name,vg_name,lv_attr,lv_size,pool_lv,origin,data_percent,metadata_percent,move_pv,mirror_log,copy_percent,convert_lv"
	lvs_cols_verbose="lv_name,vg_name,seg_count,lv_attr,lv_size,lv_major,lv_minor,lv_kernel_major,lv_kernel_minor,pool_lv,origin,data_percent,metadata_percent,move_pv,copy_percent,mirror_log,convert_lv,lv_uuid,lv_profile"
	vgs_sort="vg_name"
	vgs_cols="vg_name,pv_count,lv_count,snap_count,vg_attr,vg_size,vg_free"
	vgs_cols_verbose="vg_name,vg_attr,vg_extent_size,pv_count,lv_count,snap_count,vg_size,vg_free,vg_uuid,vg_profile"
	pvs_sort="pv_name"
	pvs_cols="pv_name,vg_name,pv_fmt,pv_attr,pv_size,pv_free"
	pvs_cols_verbose="pv_name,vg_name,pv_fmt,pv_attr,pv_size,pv_free,dev_size,pv_uuid"
	segs_sort="vg_name,lv_name,seg_start"
	segs_cols="lv_name,vg_name,lv_attr,stripes,segtype,seg_size"
	segs_cols_verbose="lv_name,vg_name,lv_attr,seg_start,seg_size,stripes,segtype,stripesize,chunksize"
	pvsegs_sort="pv_name,pvseg_start"
	pvsegs_cols="pv_name,vg_name,pv_fmt,pv_attr,pv_size,pv_free,pvseg_start,pvseg_size"
	pvsegs_cols_verbose="pv_name,vg_name,pv_fmt,pv_attr,pv_size,pv_free,pvseg_start,pvseg_size,lv_name,seg_start_pe,segtype,seg_pe_ranges"
	vgs_cols_full="vg_all"
	pvs_cols_full="pv_all"
	lvs_cols_full="lv_all"
	pvsegs_cols_full="pvseg_all,pv_uuid,lv_uuid"
	segs_cols_full="seg_all,lv_uuid"
	vgs_sort_full="vg_name"
	pvs_sort_full="pv_name"
	lvs_sort_full="vg_name,lv_name"
	pvsegs_sort_full="pv_uuid,pvseg_start"
	segs_sort_full="lv_uuid,seg_start"
	mark_hidden_devices=1
	two_word_unknown_device=0
}
dmeventd {
	raid_library="libdevmapper-event-lvm2raid.so"
	thin_library="libdevmapper-event-lvm2thin.so"
	snapshot_library="libdevmapper-event-lvm2snapshot.so"
	mirror_library="libdevmapper-event-lvm2mirror.so"
	thin_command="lvm lvextend --use-policies"
	executable="/usr/sbin/dmeventd"
}
tags {
	hosttags=0
}
local {
	system_id=""
	host_id=0
}""".strip()

LVMCONFIG2 = """
global {
	umask=077
	test=0
	units="r"
	si_unit_consistency=1
	suffix=1
	activation=1
	fallback_to_lvm1=0
}
""".strip()

LVMCONFIG3 = """
global {
	test=077
	test=0
	units="r"
	si_unit_consistency=1
	suffix=1
	activation=1
	fallback_to_lvm1=0
}
""".strip()



"""
Combiner for httpd configurations
=================================

Combiner for parsing part of httpd configurations. It collects all HttpdConf
generated from each configuration file and combines them to expose a
consolidated configuration tree.

.. note::
    At this point in time, you should **NOT** filter the httpd configurations
    to avoid finding directives in incorrect sections.
"""
from insights.core import ConfigCombiner
from insights.core.plugins import combiner
from insights.parsr.query import istartswith
from insights.parsers.httpd_conf import HttpdConf, HttpdConfSclHttpd24, HttpdConfSclJbcsHttpd24


@combiner(HttpdConf)
class HttpdConfTree(ConfigCombiner):
    """
    Exposes httpd configuration through the parsr query interface. Correctly
    handles all include directives.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """
    def __init__(self, confs):
        includes = istartswith("Include")
        super(HttpdConfTree, self).__init__(confs, "httpd.conf", includes)

    @property
    def conf_path(self):
        res = self.main.find("ServerRoot")
        return res.value if res else "/etc/httpd"


@combiner(HttpdConfSclHttpd24)
class HttpdConfSclHttpd24Tree(ConfigCombiner):
    """
    Exposes httpd configuration Software Collection httpd24 through the parsr query
    interface. Correctly handles all include directives.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """
    def __init__(self, confs):
        includes = istartswith("Include")
        super(HttpdConfSclHttpd24Tree, self).__init__(confs, "httpd.conf", includes)

    @property
    def conf_path(self):
        res = self.main.find("ServerRoot")
        return res.value if res else "/opt/rh/httpd24/root/etc/httpd"


@combiner(HttpdConfSclJbcsHttpd24)
class HttpdConfSclJbcsHttpd24Tree(ConfigCombiner):
    """
    Exposes httpd configuration Software Collection jbcs-httpd24 through the parsr query
    interface. Correctly handles all include directives.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """
    def __init__(self, confs):
        includes = istartswith("Include")
        super(HttpdConfSclJbcsHttpd24Tree, self).__init__(confs, "httpd.conf", includes)

    @property
    def conf_path(self):
        res = self.main.find("ServerRoot")
        return res.value if res else "/opt/rh/jbcs-httpd24/root/etc/httpd"



"""
DmesgLog - file ``/var/log/dmesg``
==================================

This module provides access to the messages from the kernel ring buffer
gathered from the ``/var/log/dmesg`` file.

Typical output of the ``/var/log/dmesg`` file is::

[    0.000000] Initializing cgroup subsys cpu
[    0.000000] Linux version 3.10.0-862.el7.x86_64 (mockbuild@x86-034.build.eng.bos.redhat.com) \
(gcc version 4.8.5 20150623 (Red Hat 4.8.5-28) (GCC) ) #1 SMP Wed Mar 21 18:14:51 EDT 2018
[    2.090905] SELinux:  Completing initialization.
[    2.090907] SELinux:  Setting up existing superblocks.
[    2.099684] systemd[1]: Successfully loaded SELinux policy in 82.788ms.
[    2.117410] ip_tables: (C) 2000-2006 Netfilter Core Team
[    2.117429] systemd[1]: Inserted module 'ip_tables'
[    2.376551] systemd-journald[441]: Received request to flush runtime journal from PID 1
[    2.716874] cryptd: max_cpu_qlen set to 100
[    2.804152] AES CTR mode by8 optimization enabled


Examples:
    >>> "SELinux" in dmesg_log
    True
    >>> "ip6_tables" in dmesg_log
    False
    >>> dmesg_log.get("SELinux")[0]["raw_message"]
    "[    2.090905] SELinux:  Completing initialization."
    >>> len(dmesg_log.get("SELinux"))
    2
"""

from insights import parser
from insights.specs import Specs
from insights.parsers.dmesg import DmesgLineList


@parser(Specs.dmesg_log)
class DmesgLog(DmesgLineList):
    """
    Class for parsing the ``/var/log/dmesg`` file using the DmesgLineList class.
    """
    pass



from insights.tests import context_wrap
from insights.parsers.kubelet_conf import KubeletConf


KUBELET_CONF = """
{
  "kind": "KubeletConfiguration",
  "apiVersion": "kubelet.config.k8s.io/v1beta1",
  "staticPodPath": "/etc/kubernetes/manifests",
  "syncFrequency": "0s",
  "fileCheckFrequency": "0s",
  "httpCheckFrequency": "0s",
  "tlsCipherSuites": [
    "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256",
    "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256",
    "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384",
    "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384",
    "TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256",
    "TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256"
  ],
  "tlsMinVersion": "VersionTLS12",
  "rotateCertificates": true,
  "serverTLSBootstrap": true,
  "authentication": {
    "x509": {
      "clientCAFile": "/etc/kubernetes/kubelet-ca.crt"
    },
    "webhook": {
      "cacheTTL": "0s"
    },
    "anonymous": {
      "enabled": false
    }
  },
  "authorization": {
    "webhook": {
      "cacheAuthorizedTTL": "0s",
      "cacheUnauthorizedTTL": "0s"
    }
  },
  "clusterDomain": "cluster.local",
  "clusterDNS": [
    "xx.xx.xx.xx"
  ],
  "streamingConnectionIdleTimeout": "0s",
  "nodeStatusUpdateFrequency": "10s",
  "nodeStatusReportFrequency": "5m0s",
  "imageMinimumGCAge": "2m0s",
  "imageGCHighThresholdPercent": 80,
  "imageGCLowThresholdPercent": 75,
  "volumeStatsAggPeriod": "0s",
  "systemCgroups": "/system.slice",
  "cgroupRoot": "/",
  "cgroupDriver": "systemd",
  "cpuManagerReconcilePeriod": "0s",
  "runtimeRequestTimeout": "0s",
  "maxPods": 500,
  "podPidsLimit": 4096,
  "kubeAPIQPS": 50,
  "kubeAPIBurst": 100,
  "serializeImagePulls": false,
  "evictionHard": {
    "imagefs.available": "2%",
    "imagefs.inodesFree": "2%",
    "memory.available": "200Mi",
    "nodefs.available": "2%",
    "nodefs.inodesFree": "2%"
  },
  "evictionSoft": {
    "imagefs.available": "5%",
    "imagefs.inodesFree": "5%",
    "memory.available": "500Mi",
    "nodefs.available": "5%",
    "nodefs.inodesFree": "5%"
  },
  "evictionSoftGracePeriod": {
    "imagefs.available": "1m30s",
    "imagefs.inodesFree": "1m30s",
    "memory.available": "1m30s",
    "nodefs.available": "1m30s",
    "nodefs.inodesFree": "1m30s"
  },
  "evictionPressureTransitionPeriod": "0s",
  "protectKernelDefaults": true,
  "featureGates": {
    "APIPriorityAndFairness": true,
    "DownwardAPIHugePages": true,
    "RetroactiveDefaultStorageClass": false,
    "RotateKubeletServerCertificate": true
  },
  "memorySwap": {},
  "containerLogMaxSize": "50Mi",
  "logging": {
    "flushFrequency": 0,
    "verbosity": 0,
    "options": {
      "json": {
        "infoBufferSize": "0"
      }
    }
  },
  "shutdownGracePeriod": "0s",
  "shutdownGracePeriodCriticalPods": "0s"
}
""".strip()


def test_kubelet_conf():
    conf = KubeletConf(context_wrap(KUBELET_CONF))
    assert conf.data['kind'] == "KubeletConfiguration"
    assert len(conf.data.keys()) == 42
    assert conf.data['featureGates']['DownwardAPIHugePages'] is True



"""
Parsers for file ``/sys/kernel/debug/x86/*_enabled`` outputs
============================================================

This module provides the following parsers:

X86PTIEnabled - file ``/sys/kernel/debug/x86/pti_enabled``
----------------------------------------------------------

X86IBPBEnabled - file ``/sys/kernel/debug/x86/ibpb_enabled``
------------------------------------------------------------

X86IBRSEnabled - file ``/sys/kernel/debug/x86/ibrs_enabled``
------------------------------------------------------------

X86RETPEnabled - file ``/sys/kernel/debug/x86/retp_enabled``
------------------------------------------------------------

"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


class X86DebugEnabled(Parser):
    """
    Class for parsing file ``/sys/kernel/debug/x86/*_enabled``

    Attributes:
        value (int): the result parsed of `/sys/kernel/debug/x86/*_enabled`

    Raises:
        SkipComponent: When input content is empty
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("Input content is empty")
        # it is a digit
        self.value = int(content[0])


@parser(Specs.x86_ibpb_enabled)
class X86IBPBEnabled(X86DebugEnabled):
    """
    Class for parsing file ``/sys/kernel/debug/x86/ibpb_enabled``
    Typical output of file ``/sys/kernel/debug/x86/retp_enabled`` looks like::

        1

    Examples:
        >>> type(dva)
        <class 'insights.parsers.x86_debug.X86IBPBEnabled'>
        >>> dva.value
        1

    Attributes:
        value (int): the result parsed of '/sys/kernel/debug/x86/ibpb_enabled'

    Raises:
        SkipComponent: When input content is empty
    """
    pass


@parser(Specs.x86_ibrs_enabled)
class X86IBRSEnabled(X86DebugEnabled):
    """
    Class for parsing file ``/sys/kernel/debug/x86/ibrs_enabled``
    Typical output of file ``/sys/kernel/debug/x86/ibrs_enabled`` looks like::

        0

    Examples:
        >>> type(dl)
        <class 'insights.parsers.x86_debug.X86IBRSEnabled'>
        >>> dl.value
        1

    Attributes:
        value (int): the result parsed of '/sys/kernel/debug/x86/ibrs_enabled'

    Raises:
        SkipComponent: When input content is empty
    """
    pass


@parser(Specs.x86_pti_enabled)
class X86PTIEnabled(X86DebugEnabled):
    """
    Class for parsing file ``/sys/kernel/debug/x86/pti_enabled``
    Typical output of file ``/sys/kernel/debug/x86/pti_enabled`` looks like::

        0

    Examples:
        >>> type(dv)
        <class 'insights.parsers.x86_debug.X86PTIEnabled'>
        >>> dv.value
        1

    Attributes:
        value (int): the result parsed of '/sys/kernel/debug/x86/pti_enabled'

    Raises:
        SkipComponent: When input content is empty
    """
    pass


@parser(Specs.x86_retp_enabled)
class X86RETPEnabled(X86DebugEnabled):
    """
    Class for parsing file ``/sys/kernel/debug/x86/retp_enabled``
    Typical output of file ``/sys/kernel/debug/x86/retp_enabled`` looks like::

        1

    Examples:
        >>> type(dval)
        <class 'insights.parsers.x86_debug.X86RETPEnabled'>
        >>> dval.value
        1

    Attributes:
        value (int): the result parsed of '/sys/kernel/debug/x86/retp_enabled'

    Raises:
        SkipComponent: When input content is empty
    """
    pass



"""
Configuration Parsers for Krb5
==============================

Below parsers are included:

Krb5Configuration - files ``/etc/krb5.conf`` and ``/etc/krb5.conf.d/*``
-----------------------------------------------------------------------

Krb5LocalauthPlugin - file ``/var/lib/sss/pubconf/krb5.include.d/localauth_plugin``
-----------------------------------------------------------------------------------
"""

from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util import parse_bool


def _handle_key_value(t_dict, key, value):
    """
    Function to handle key has multi value, and return the values as list.
    """
    if key in t_dict:
        val = t_dict[key]
        if isinstance(val, str):
            val = [val]
        val.append(value)
        return val
    return value


class Krb5ConfBase(Parser, dict):
    """
    Base Class to process the Kerberos relevant configurations.

    The Kerberos Configuration are generally in .ini format. it is like an
    ordinary .ini file except that values can include a multiple line key-value
    pair 'relation' that starts with a '{' and end with a '}' on a trailing line.
    So we track whether we're in curly braces by setting `is_squ` when we enter
    a relation, and clearing it when we leave.  Please fill in the remainder of
    the logic here.

    """

    RESERVED_ATTRS = None

    def parse_content(self, content):
        if self.RESERVED_ATTRS is None:
            raise RuntimeError('Not applied when "RESERVED_ATTRS" does not exist')
        for key in self.RESERVED_ATTRS:
            setattr(self, key, [])
        dict_all = {}
        is_squ = False
        section_name = ""
        squ_value = {}
        squ_section_name = ""
        section_value = {}
        unchangeable_tags = []
        for line in content:
            line = line.strip()
            if not line or line.startswith('#'):
                # skip empty and commented lines
                continue
            if line.startswith(self.RESERVED_ATTRS):
                key, value = [i.strip() for i in line.split(None, 1)]
                if key in self.RESERVED_ATTRS:
                    getattr(self, key).append(value)
                    continue
            if is_squ:
                # If in {} sub_section, get the key_value pair
                if "=" in line:
                    key, value = [i.strip() for i in line.split('=', 1)]
                    if key not in unchangeable_tags:
                        value = value.split()[0] if value else value
                        squ_value[key] = _handle_key_value(squ_value, key, value)
                    if line.endswith("*"):
                        unchangeable_tags.append(key)
                # The {} sub_section should end with },
                # if it is, set the whole value to the sub_section name,
                # and clean the flag
                else:
                    section_value[squ_section_name] = squ_value
                    is_squ = False
                    squ_section_name = ""
                    squ_value = {}
            else:
                # [XXX] means a section, get the section name and the value
                # format is dict.
                if line.startswith("[") and line.endswith("]"):
                    # If first section, just get the section name,
                    # if not, set the value to the former section and
                    # get the section name
                    section_name = line.strip("[]")
                    section_value = {}
                    if section_name:
                        dict_all[section_name] = section_value
                # key value format is XXX = YYY, store as dict
                elif "=" in line and not line.endswith("{"):
                    key, value = [i.strip() for i in line.split('=', 1)]
                    if key not in unchangeable_tags:
                        value = value.split()[0] if value else value
                        section_value[key] = _handle_key_value(section_value, key, value)
                    if line.endswith("*"):
                        unchangeable_tags.append(key)
                # The {} sub_section should start with format XXX = {
                else:
                    is_squ = True
                    squ_section_name = line.split("=")[0].strip()
        if dict_all:
            self.update(dict_all)
        elif not any(getattr(self, key) for key in self.RESERVED_ATTRS):
            raise SkipComponent()

    @property
    def data(self):
        """
        Keep backward compatibility. The "data" atrribute is deprecated,
        the parser itself is dictionary.

        .. warning::
            This will be removed from 3.8.0.
        """
        return self

    def sections(self):
        """
        Return a list of section names.
        """
        return sorted(self.keys())

    def has_section(self, section):
        """
        Indicate whether the named section is present in the configuration.
        Return True if the given section is present, and False if not present.
        """
        return section in self

    def options(self, section):
        """
        Return a list of option names for the given section name.
        """
        return sorted(self[section].keys()) if self.has_section(section) else []

    def has_option(self, section, option):
        """
        Check for the existence of a given option in a given section.
        Return True if the given option is present, and False if not present.
        """
        return False if section not in self else option in self[section]

    def getboolean(self, section, option):
        """Parse option as bool

        Returns None is not a krb5.conf boolean string.
        """
        return parse_bool(self[section][option], default=None)


@parser(Specs.krb5)
class Krb5Configuration(Krb5ConfBase):
    """
    Krb5 Configuration are ``/etc/krb5.conf`` and ``/etc/krb5.conf.d/*``.

    See :class:`Krb5ConfBase` for details.

    Attributes:
        includedir (list): The directory list that `krb5.conf` includes via
            `includedir` directive
        include (list): The configuration file list that `krb5.conf` includes
            via `include` directive
        module (list): The module list that `krb5.conf` specifed via `module`
            directive

    Sample content::

        include /etc/krb5test.conf
        [realms]
          dns_lookup_realm = false
          ticket_lifetime = 24h
          default_ccache_name = KEYRING:persistent:%{uid}
          EXAMPLE.COM = {
           kdc = kerberos.example.com
           admin_server = kerberos.example.com
          }
          pam = {
           debug = false
           krb4_convert = false
           ticket_lifetime = 36000
          }
        [libdefaults]
          dns_lookup_realm = false
          dnsdsd = false
          ticket_lifetime = 24h
          EXAMPLE.COM = {
           kdc = kerberos2.example.com
           admin_server = kerberos2.example.com
         }

    Example:
        >>> type(krb5_conf)
        <class 'insights.parsers.krb5.Krb5Configuration'>
        >>> krb5_conf["libdefaults"]["dnsdsd"]
        'false'
        >>> krb5_conf["realms"]["EXAMPLE.COM"]["kdc"]
        'kerberos.example.com'
        >>> krb5_conf.sections()
        ['libdefaults', 'realms']
        >>> krb5_conf.has_section("realms")
        True
        >>> krb5_conf.has_option("realms", "nosuchoption")
        False
        >>> krb5_conf.options("libdefaults")
        ['EXAMPLE.COM', 'dns_lookup_realm', 'dnsdsd', 'ticket_lifetime']
        >>> krb5_conf.include
        ['/etc/krb5test.conf']
    """

    RESERVED_ATTRS = ("includedir", "include", "module")


@parser(Specs.krb5_localauth_plugin)
class Krb5LocalauthPlugin(Krb5ConfBase):
    """
    Krb5 Configuration parser for `/var/lib/sss/pubconf/krb5.include.d/localauth_plugin`

    Sample input::

        [plugins]
         localauth = {
          module = sssd:/usr/lib64/sssd/modules/sssd_krb5_localauth_plugin.so
         }

    Examples:
        >>> type(krb5_LP)
        <class 'insights.parsers.krb5.Krb5LocalauthPlugin'>
        >>> krb5_LP['plugins']['localauth']['module']
        'sssd:/usr/lib64/sssd/modules/sssd_krb5_localauth_plugin.so'
    """

    RESERVED_ATTRS = ()



import errno
import os.path
import logging
import shutil
import tempfile
import subprocess

logger = logging.getLogger(__name__)


class GPGCommandResult(object):
    """Output of a GPGCommand.

    :param ok: Result of an operation.
    :type ok: bool
    :param return_code: Return code of an operation.
    :type return_code: int
    :param stdout: Standard output of the command.
    :type stdout: str
    :param stderr: Standard error of the command.
    :type stderr: str
    :param _command: An optional reference to the GPGCommand object that created the result.
    :type _command: GPGCommand | None
    """
    def __init__(self, ok, return_code, stdout, stderr, command):
        self.ok = ok
        self.return_code = return_code
        self.stdout = stdout
        self.stderr = stderr
        self._command = command

    def __str__(self):
        return (
            "<{cls} ok={ok} return_code={code} stdout={out} stderr={err}>"
        ).format(
            cls=self.__class__.__name__,
            ok=self.ok, code=self.return_code,
            out=self.stdout, err=self.stderr,
        )


class GPGCommand(object):
    """GPG command run in a temporary environment.

    :param command: The command to be executed.
    :type command: list[str]
    :param key: Path to the GPG public key to check against.
    :type key: str
    :param _home: Path to the temporary GPG home directory.
    :type _home: str | None
    :param _raw_command: The last invoked command.
    :type _raw_command: list[str] | None
    """
    TEMPORARY_GPG_HOME_PARENT_DIRECTORY = "/var/lib/insights/"

    def __init__(self, command, key):
        self.command = command
        self.key = key
        self._home = None
        self._raw_command = None

    def __str__(self):
        return "<{cls} _home={home} _raw_command={raw}>".format(
            cls=self.__class__.__name__,
            home=self._home,
            raw=self._raw_command
        )

    def _setup(self):
        """Prepare GPG environment.

        :returns: Result of the GPG key import.
        :rtype: GPGCommandResult
        """
        # The /var/lib/insights/ directory is used instead of /tmp/ because
        # GPG needs to have RW permissions in it, and existing SELinux rules only
        # allow access here.
        self._home = tempfile.mkdtemp(dir=type(self).TEMPORARY_GPG_HOME_PARENT_DIRECTORY)

        logger.debug("setting up gpg temporary environment in '{home}'".format(home=self._home))
        result = self._run(["--import", self.key])  # type: GPGCommandResult
        if not result.ok:
            logger.warning("failed to import key '{key}': {result}".format(
                key=self.key, result=result
            ))
        return result

    def _supports_cleanup_socket(self):
        """Queries for the version of GPG binary.

        :returns: `True` if the gnupg is known for supporting `--kill all`.
        :rtype: bool
        """
        version_process = subprocess.Popen(
            ["/usr/bin/gpg", "--version"],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            universal_newlines=True,
            env={"GNUPGHOME": self._home, "LC_ALL": "C.UTF-8"},
        )
        stdout, stderr = version_process.communicate()
        if version_process.returncode != 0:
            stderr = stderr.decode("utf-8") if isinstance(stderr, bytes) else stderr
            stderr = "\n".join("stderr: {line}".format(line=line) for line in stderr.split("\n") if len(line))
            logger.debug("could not query for gpg version:\n{err}".format(err=stderr))
            return False

        for line in stdout.split("\n"):
            if line.startswith("gpg (GnuPG) "):
                version = line.split(" ")[-1]  # type: str
                break
        else:
            stdout = "\n".join("stdout: {line}".format(line=line) for line in stdout.split("\n") if len(line))
            logger.debug("could not query for gpg version: output not recognized:\n{out}".format(out=stdout))
            return False

        try:
            version_info = tuple(int(v) for v in version.split("."))
        except Exception as exc:
            logger.debug("gpg version could not be parsed: '{version}: {exc}".format(version=version, exc=str(exc)))
            return False
        if len(version_info) < 3:
            logger.debug("gpg version is not recognized: '{version}'".format(version=version))
            return False

        # `gpgconf --kill` was added in GnuPG 2.1.0-beta2 and `--kill all` exists since 2.1.18.
        # - 2.1.0b1: commit 7c03c8cc65e68f1d77a5a5a497025191fe5df5e9 in GPG's repository.
        # - 2.1.18: https://lists.gnupg.org/pipermail/gnupg-announce/2017q1/000401.html
        #
        # RHEL versions come with the following gpg versions:
        # - 6.10: 2.0.14
        # - 7.9:  2.0.22
        # - 8.9:  2.2.20
        # - 9.3:  2.3.3
        # which means this code should return `True` for RHEL 8 and above.
        return version_info >= (2, 1, 18)

    def _cleanup_socket(self):
        """Stop GPG socket in its home directory."""
        # GPG writes a temporary socket file for the gpg-agent into the home
        # directory. This is only supported since gnupg 2.1.18 (RHEL 8).
        shutdown_process = subprocess.Popen(
            ["/usr/bin/gpgconf", "--kill", "all"],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            universal_newlines=True,
            env={"GNUPGHOME": self._home, "LC_ALL": "C.UTF-8"},
        )
        _, stderr = shutdown_process.communicate()
        if shutdown_process.returncode != 0:
            stderr = "\n".join("stderr: {line}".format(line=line) for line in stderr.split("\n") if len(line))
            logger.debug(
                "could not kill the GPG agent, got return code {rc}: \n{err}".format(
                    rc=shutdown_process.returncode, err=stderr
                )
            )

    def _cleanup(self):
        """Clean up GPG environment."""
        if self._supports_cleanup_socket():
            self._cleanup_socket()

        # Older systems do not support `gpgconf --kill`.
        # The socket may remove its socket file after `rmtree()` has determined
        # it should be deleted, but before the actual deletion occurs.
        # This would cause a FileNotFoundError/OSError.
        for _ in range(5):
            try:
                shutil.rmtree(self._home)
                break
            except OSError as exc:
                if exc.errno == errno.ENOENT:
                    # The file has already been removed by the `gpg-agent`.
                    continue
                raise
        else:
            logger.debug(
                "could not clean up temporary GPG directory '{path}'".format(path=self._home)
            )

    def _run(self, command):
        """Run the actual command.

        :returns: The result of the shell command.
        :rtype: GPGCommandResult
        """
        self._raw_command = ["/usr/bin/gpg", "--homedir", self._home] + command
        process = subprocess.Popen(
            self._raw_command,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            env={"LC_ALL": "C.UTF-8"},
        )
        stdout, stderr = process.communicate()

        result = GPGCommandResult(
            ok=process.returncode == 0,
            return_code=process.returncode,
            stdout=stdout.decode("utf-8"),
            stderr=stderr.decode("utf-8"),
            command=self,
        )

        if result.ok:
            logger.debug("gpg command {command}: ok".format(command=command))
        else:
            logger.debug(
                "gpg command {command} returned non-zero code: {result}".format(
                    command=command, result=result,
                )
            )

        return result

    def evaluate(self):
        """Run the command.

        :returns: The result of the shell command.
        :rtype: GPGCommandResult
        """
        try:
            result = self._setup()
            if not result.ok:
                logger.debug("gpg setup failed")
                return result

            logger.debug("running gpg in the temporary environment")
            return self._run(self.command)
        finally:
            self._cleanup()


def verify_gpg_signed_file(file, signature, key):
    """
    Verify a file that was signed using GPG.

    :param file: A path to the signed file.
    :type file: str
    :param signature: A path to the detached signature.
    :type signature: str
    :param key: Path to the public GPG key on the filesystem to check against.
    :type key: str

    :returns: Evaluated GPG command.
    :rtype: GPGCommandResult
    """
    if not os.path.isfile(file):
        logger.debug(
            "cannot verify signature of '{file}', file does not exist".format(
                file=file
            )
        )
        return GPGCommandResult(
            ok=False,
            return_code=1,
            stdout="",
            stderr="file '{path}' does not exist".format(path=file),
            command=None,
        )

    if not os.path.isfile(signature):
        logger.debug((
            "cannot verify signature of '{file}', "
            "signature '{signature}' does not exist"
        ).format(file=file, signature=signature))
        return GPGCommandResult(
            ok=False,
            return_code=1,
            stdout="",
            stderr="file '{path}' does not exist".format(path=signature),
            command=None,
        )

    gpg = GPGCommand(command=["--verify", signature, file], key=key)

    logger.debug("starting gpg verification process for '{file}'".format(file=file))
    result = gpg.evaluate()  # type: GPGCommandResult

    if result.ok:
        logger.debug("signature verification of '{file}' passed".format(file=file))
    else:
        logger.debug("signature verification of '{file}' failed".format(file=file))

    return result



"""
CrictlPs - command ``crictl ps --quiet``
========================================

This parser reads the output of ``crictl ps --quiet`` command and returns a list of
container records with their associated metadata. The parser handles various container
states and time formats in the "created" field.

The parser is designed to handle the complex structure of crictl output where the
"created" field may contain spaces (e.g., "About a minute ago", "2 hours ago") and
uses intelligent parsing to correctly separate fields.
"""  # noqa: E501

from insights.core import CommandParser
from insights.core.plugins import parser
from insights.specs import Specs
from insights.core.exceptions import ParseException, SkipComponent


@parser(Specs.crictl_ps)
class CrictlPs(CommandParser, list):
    """
    Parser for the output of ``crictl ps --quiet`` command.

    The parser is designed to handle the complex structure of crictl output where the
    "created" field may contain spaces (e.g., "About a minute ago", "2 hours ago") and
    uses intelligent parsing to correctly separate fields.

    Sample input::

        CONTAINER           IMAGE                                                                                                                        CREATED              STATE               NAME                                          ATTEMPT             POD ID              POD
        93b10093a8263       bea2d277eb71530a376a68be9760260cedb59f2392bb6e7793b05d5350df8d4c                                                             About a minute ago   Running             oauth-apiserver                               185                 19d971fe5c478       apiserver-7cd97c59ff-dwckz
        e34ce05ade472       2c96c7c72cf99490b4bdbb7389020b7e4b5bb7dc43ea9cadc4d5af43cb300b3f                                                             9 days ago           Running             guard                                         1                   c48cc19e5b0b1       etcd-guard-nah-4jnq5-master-v8z5h-0
        471d75b135b5b       90e50eece96ef2a252b729a76a2ee3360d3623295cceb7d3e623b55cb7aef30a                                                             9 days ago           Running             etcd                                          39                  d2dd84f8db754       etcd-nah-4jnq5-master-v8z5h-0

    The parser returns a list of dictionaries, each containing one container record with the
    following fields:
    - container_id (str): The container ID (e.g., '93b10093a8263')
    - image (str): The container image hash or reference (e.g., 'bea2d277eb71530a...')
    - created (str): When the container was created (e.g., 'About a minute ago', '9 days ago')
    - state (str): Current container state (e.g., 'Running', 'Exited', 'ContainerCreating', 'Unknown')
    - name (str): Container name (e.g., 'oauth-apiserver')
    - attempt (str): Container restart attempt number (e.g., '185')
    - pod_id (str): Associated pod ID (e.g., '19d971fe5c478')
    - pod (str): Associated pod name (e.g., 'apiserver-7cd97c59ff-dwckz')

    Supported Container States:
    - Running: Container is currently running
    - Exited: Container has exited
    - Created: Container has been created but not started
    - ContainerCreating: Container is in the process of being created
    - Unknown: Container state is unknown
    - Pending: Container is pending (waiting for resources)

    Examples:

        Basic usage:
        >>> crictl_ps = CrictlPs(context)
        >>> len(crictl_ps)
        3
        >>> crictl_ps[0]['container_id']
        '93b10093a8263'
        >>> crictl_ps[0]['image']
        'bea2d277eb71530a376a68be9760260cedb59f2392bb6e7793b05d5350df8d4c'
        >>> crictl_ps[0]['created']
        'About a minute ago'
        >>> crictl_ps[0]['state']
        'Running'
        >>> crictl_ps[0]['name']
        'oauth-apiserver'
        >>> crictl_ps[0]['attempt']
        '185'
        >>> crictl_ps[0]['pod_id']
        '19d971fe5c478'
        >>> crictl_ps[0]['pod']
        'apiserver-7cd97c59ff-dwckz'

    Raises:
        ParseException: If the input content is invalid or doesn't contain the expected
            header line with "CONTAINER" keyword.
        SkipComponent: If no container records are found after parsing.
    """

    def parse_content(self, content):
        """
        Parse the crictl ps output content.

        Args:
            content (list): List of strings representing the crictl ps output lines.

        Raises:
            ParseException: If the content is invalid or doesn't contain the expected header.
            SkipComponent: If no container records are found after parsing.
        """
        if len(content) < 1 or "CONTAINER" not in content[0]:
            raise ParseException("invalid content: {0}".format(content) if content else 'empty file')

        # Skip the header line
        for line in content[1:]:
            line = line.strip()
            if not line:
                continue

            # Parse each line manually to handle the space-containing "created" field
            parsed = self._parse_line(line)
            if parsed:
                self.append(parsed)

        # Raise SkipComponent if no containers were found
        if not self:
            raise SkipComponent("No container records found")

    def _parse_line(self, line):
        """
        Parse a single line of crictl ps output.

        This method handles the complex parsing logic required for crictl output
        where the "created" field may contain spaces and the image field can be
        very long.

        Args:
            line (str): A single line from crictl ps output.

        Returns:
            dict: Parsed container record with all fields, or None if parsing fails.

        Note:
            The parsing logic:
            1. Splits the line by whitespace
            2. First two fields are always container_id and image
            3. Finds the state field by looking for known state keywords
            4. Everything between image and state is the "created" field
            5. Remaining fields are mapped in order: name, attempt, pod_id, pod
        """
        # Split by whitespace
        parts = line.split()

        if len(parts) < 8:
            return None

        # The structure is:
        # container_id image created... state name attempt pod_id pod

        # Find the state field (it's always a single word like "Running", "Exited", etc.)
        # Look for common state values
        state_keywords = ['Running', 'Exited', 'Created', 'Unknown', 'ContainerCreating', 'Pending']
        state_index = None
        for i, part in enumerate(parts[2:], 2):
            if part in state_keywords:
                state_index = i
                break

        if state_index is None:
            return None

        # The remaining parts after state
        remaining = parts[state_index + 1:]

        if len(remaining) < 4:
            return None

        return {
            'container_id': parts[0],  # First two parts are always container_id and image
            'image': parts[1],
            'created': ' '.join(parts[2:state_index]),  # Everything between image and state is the "created" field
            'state': parts[state_index],  # The state is at state_index
            'name': remaining[-4],  # The last 4 parts are: name, attempt, pod_id, pod
            'attempt': remaining[-3],
            'pod_id': remaining[-2],
            'pod': remaining[-1]
        }

    @property
    def container_ids(self):
        return [record['container_id'] for record in self]



import os
from tempfile import mkdtemp
from insights import dr
from insights.core.plugins import component
from insights.core.serde import Hydration
from insights.core.spec_factory import (
    RawFileProvider,
    TextFileProvider,
)
from insights.util import fs

root = os.path.dirname(__file__)
relative_path = os.path.basename(__file__)


@component()
def thing():
    pass


def test_text_file():
    before = TextFileProvider(relative_path, root)
    broker = dr.Broker()
    broker[thing] = before

    tmp_path = mkdtemp()
    try:
        hydra = Hydration(tmp_path)
        hydra.dehydrate(thing, broker)
        after = hydra.hydrate()[thing]
        assert after.content == before.content
    finally:
        if tmp_path and os.path.exists(tmp_path):
            fs.remove(tmp_path)


def test_raw_file():
    before = RawFileProvider(relative_path, root)
    broker = dr.Broker()
    broker[thing] = before

    tmp_path = mkdtemp()
    try:
        hydra = Hydration(tmp_path)
        hydra.dehydrate(thing, broker)
        after = hydra.hydrate()[thing]
        assert after.content == before.content
    finally:
        if tmp_path and os.path.exists(tmp_path):
            fs.remove(tmp_path)



from insights.combiners.grubby import Grubby, IsUEFIBoot
from insights.core.exceptions import ParseException
from insights.parsers.grubby import GrubbyInfoAll, GrubbyDefaultIndex
from insights.parsers.grubenv import GrubEnv
from insights.parsers.ls import LSlanFiltered
from insights.tests import context_wrap
import pytest

DEFAULT_INDEX_1 = '0'
DEFAULT_INDEX_2 = '3'

GRUBBY_INFO_ALL = """
index=0
kernel="/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-5.14.0-162.6.1.el9_1.x86_64.img"
title="Red Hat Enterprise Linux (5.14.0-162.6.1.el9_1.x86_64) 9.1 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-5.14.0-162.6.1.el9_1.x86_64"
index=1
kernel="/boot/vmlinuz-5.14.0-70.13.1.el9_0.x86_64"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-5.14.0-70.13.1.el9_0.x86_64.img"
title="Red Hat Enterprise Linux (5.14.0-70.13.1.el9_0.x86_64) 9.0 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-5.14.0-70.13.1.el9_0.x86_64"
index=2
kernel="/boot/vmlinuz-0-rescue-4d684a4a6166439a867e701ded4f7e10"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-0-rescue-4d684a4a6166439a867e701ded4f7e10.img"
title="Red Hat Enterprise Linux (0-rescue-4d684a4a6166439a867e701ded4f7e10) 9.0 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-0-rescue"
""".strip()

GRUBBY_INFO_ALL_AB_1 = """
index=0
title="Red Hat Enterprise Linux (5.14.0-162.6.1.el9_1.x86_64) 9.1 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-5.14.0-162.6.1.el9_1.x86_64"
""".strip()

GRUBBY_INFO_ALL_AB_2 = """
index=0
kernel="/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb intel_iommu=on"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-5.14.0-162.6.1.el9_1.x86_64.img"
title="Red Hat Enterprise Linux (5.14.0-162.6.1.el9_1.x86_64) 9.1 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-5.14.0-162.6.1.el9_1.x86_64"
index=1
kernel=""
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet retbleed=stuff"
root="/dev/mapper/rhel-root"
initrd=""
title="Red Hat Enterprise Linux (5.14.0-70.13.1.el9_0.x86_64) 9.0 (Plow)"
id="4d684a4a6166439a867e701ded4f7e10-5.14.0-70.13.1.el9_0.x86_64"
""".strip()

GRUBBY_INFO_ALL_WITH_PARAMS = """
index=0
kernel="/boot/vmlinuz-5.14.0-570.30.1.el9_6.x86_64"
args="ro audit=1 crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M rd.lvm.lv=rhel/root rhgb quiet selinux=0 audit_backlog_limit=8192 $tuned_params"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-5.14.0-570.30.1.el9_6.x86_64.img $tuned_initrd"
title="Red Hat Enterprise Linux (5.14.0-570.30.1.el9_6.x86_64) 9.6 (Plow)"
id="1234567890abcdefffffffffffffffff-5.14.0-570.30.1.el9_6.x86_64"
index=1
kernel="/boot/vmlinuz-0-rescue-1234567890abcdefffffffffffffffff"
args="ro audit=1 $kernelopts $tuned_params"
root="/dev/mapper/rhel-root"
initrd="/boot/initramfs-0-rescue-1234567890abcdefffffffffffffffff.img"
title="Red Hat Enterprise Linux (0-rescue-1234567890abcdefffffffffffffffff) 9.6 (Plow)"
id="1234567890abcdefffffffffffffffff-0-rescue"
""".strip()

GRUBENV_WITH_PARAMS = """
saved_entry=1234567890abcdefffffffffffffffff-5.14.0-570.30.1.el9_6.x86_64
menu_auto_hide=1
boot_success=1
boot_indeterminate=0
kernelopts=console=tty0 console=ttyS0,115200 noapic
tuned_params=transparent_hugepages=never
tuned_initrd=foo=bar
""".strip()

GRUBENV_WITHOUT_PARAMS = """
saved_entry=1234567890abcdefffffffffffffffff-5.14.0-570.30.1.el9_6.x86_64
menu_auto_hide=1
boot_success=1
boot_indeterminate=0
""".strip()

LS_SYS_FIRMWARE_NOT_EFI = """
/sys/firmware:
total 0
drwxr-xr-x  6 0 0 0 Aug 19 14:57 dmi
""".strip()

LS_SYS_FIRMWARE_IS_EFI = """
/sys/firmware:
total 0
drwxr-xr-x  6 0 0 0 Aug 19 14:57 efi
""".strip()


def test_grubby():
    grubby_info_all = GrubbyInfoAll(context_wrap(GRUBBY_INFO_ALL))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1))
    ls_lan = LSlanFiltered(context_wrap(LS_SYS_FIRMWARE_NOT_EFI))
    is_uefi_boot = IsUEFIBoot(ls_lan, None)
    result = Grubby(grubby_info_all, grubby_default_index, None, is_uefi_boot)

    assert result.default_index == 0
    default_boot_entry_data = dict(
        index=0,
        kernel="/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64",
        args={
            'ro': [True],
            'crashkernel': ['1G-4G:192M,4G-64G:256M,64G-:512M'],
            'resume': ['/dev/mapper/rhel-swap'],
            'rd.lvm.lv': ['rhel/root', 'rhel/swap'],
            'rhgb': [True],
            'quiet': [True],
            'retbleed': ['stuff'],
            'root': ['/dev/mapper/rhel-root'],
            '/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64': [True],
        },
        cmdline="/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64 root=/dev/mapper/rhel-root ro "
        "crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M "
        "resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap "
        "rhgb quiet retbleed=stuff",
        root="/dev/mapper/rhel-root",
        initrd="/boot/initramfs-5.14.0-162.6.1.el9_1.x86_64.img",
        title="Red Hat Enterprise Linux (5.14.0-162.6.1.el9_1.x86_64) 9.1 (Plow)",
        name="Red Hat Enterprise Linux (5.14.0-162.6.1.el9_1.x86_64) 9.1 (Plow)",
        id="4d684a4a6166439a867e701ded4f7e10-5.14.0-162.6.1.el9_1.x86_64",
    )
    for k, v in result.default_boot_entry.items():
        assert v == default_boot_entry_data[k]
    assert result.default_kernel == "/boot/vmlinuz-5.14.0-162.6.1.el9_1.x86_64"
    assert len(result.boot_entries) == 3
    assert len(result.kernel_initrds['grub_kernels']) == 3
    assert result.is_kdump_iommu_enabled is False
    assert not result.is_efi


def test_grubby_ab():
    grubby_info_all = GrubbyInfoAll(context_wrap(GRUBBY_INFO_ALL))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_2))
    with pytest.raises(ParseException) as excinfo:
        Grubby(grubby_info_all, grubby_default_index, None, None)
    assert "DEFAULT index 3 not exist in parsed boot_entries: [0, 1, 2]" in str(excinfo.value)

    grubby_info_all = GrubbyInfoAll(context_wrap(GRUBBY_INFO_ALL_AB_1))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1))
    with pytest.raises(ParseException) as excinfo:
        Grubby(grubby_info_all, grubby_default_index, None, None)
    assert "DEFAULT kernel-path not exist in default-index: 0" in str(excinfo.value)

    grubby_info_all = GrubbyInfoAll(context_wrap(GRUBBY_INFO_ALL_AB_2))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1))
    result = Grubby(grubby_info_all, grubby_default_index, None, None)
    assert len(result.kernel_initrds['grub_kernels']) == 1
    assert len(result.kernel_initrds['grub_initrds']) == 1  # cov test
    assert result.is_kdump_iommu_enabled is True
    assert result.is_kdump_iommu_enabled is True  # cov test


def test_grubby_extend_with_grubenv():
    grubby_info_all = GrubbyInfoAll(context_wrap(GRUBBY_INFO_ALL_WITH_PARAMS))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(DEFAULT_INDEX_1))
    grubenv = GrubEnv(context_wrap(GRUBENV_WITH_PARAMS))
    ls_lan = LSlanFiltered(context_wrap(LS_SYS_FIRMWARE_IS_EFI))
    is_uefi_boot = IsUEFIBoot(ls_lan, None)

    result = Grubby(grubby_info_all, grubby_default_index, grubenv, is_uefi_boot)
    assert result.default_index == 0
    default_boot_entry_data = dict(
        index=0,
        kernel="/boot/vmlinuz-5.14.0-570.30.1.el9_6.x86_64",
        args={
            'audit': ['1'],
            'audit_backlog_limit': ['8192'],
            'crashkernel': ['1G-4G:192M,4G-64G:256M,64G-:512M'],
            'quiet': [True],
            'rd.lvm.lv': ['rhel/root'],
            'rhgb': [True],
            'ro': [True],
            'root': ['/dev/mapper/rhel-root'],
            'selinux': ['0'],
            'transparent_hugepages': ['never'],
            '/boot/vmlinuz-5.14.0-570.30.1.el9_6.x86_64': [True],
        },
        cmdline='/boot/vmlinuz-5.14.0-570.30.1.el9_6.x86_64 root=/dev/mapper/rhel-root ro audit=1 '
        'crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M rd.lvm.lv=rhel/root '
        'rhgb quiet selinux=0 audit_backlog_limit=8192 '
        'transparent_hugepages=never',
        root="/dev/mapper/rhel-root",
        initrd="/boot/initramfs-5.14.0-570.30.1.el9_6.x86_64.img foo=bar",
        title="Red Hat Enterprise Linux (5.14.0-570.30.1.el9_6.x86_64) 9.6 (Plow)",
        name="Red Hat Enterprise Linux (5.14.0-570.30.1.el9_6.x86_64) 9.6 (Plow)",
        id='1234567890abcdefffffffffffffffff-5.14.0-570.30.1.el9_6.x86_64',
    )
    for k, v in result.default_boot_entry.items():
        assert v == default_boot_entry_data[k]
    assert result.default_kernel == "/boot/vmlinuz-5.14.0-570.30.1.el9_6.x86_64"
    assert result.is_efi
    assert len(result.boot_entries) == 2
    assert (
        result.boot_entries[1]['initrd']
        == "/boot/initramfs-0-rescue-1234567890abcdefffffffffffffffff.img"
    )
    assert result.boot_entries[1]['args'] == {
        'root': ['/dev/mapper/rhel-root'],
        'ro': [True],
        'audit': ['1'],
        'console': ['tty0', 'ttyS0,115200'],
        'noapic': [True],
        'transparent_hugepages': ['never'],
        '/boot/vmlinuz-0-rescue-1234567890abcdefffffffffffffffff': [True],
    }
    assert result.boot_entries[1]['cmdline'] == (
        "/boot/vmlinuz-0-rescue-1234567890abcdefffffffffffffffff "
        "root=/dev/mapper/rhel-root ro audit=1 console=tty0 "
        "console=ttyS0,115200 noapic transparent_hugepages=never"
    )

    # without grubenv
    result = Grubby(grubby_info_all, grubby_default_index, None, None)
    assert result.default_index == 0
    assert (
        result.default_boot_entry["initrd"]
        == "/boot/initramfs-5.14.0-570.30.1.el9_6.x86_64.img $tuned_initrd"
    )
    assert result.default_boot_entry["args"]["$tuned_params"] == [True]
    assert result.default_boot_entry["cmdline"].endswith("audit_backlog_limit=8192 $tuned_params")
    assert len(result.boot_entries) == 2
    assert not result.is_efi

    # with grubenv w/o params setting
    grubenv_wo_params = GrubEnv(context_wrap(GRUBENV_WITHOUT_PARAMS))
    result = Grubby(grubby_info_all, grubby_default_index, grubenv_wo_params, None)
    assert result.default_index == 0
    assert result.default_boot_entry["initrd"] == "/boot/initramfs-5.14.0-570.30.1.el9_6.x86_64.img"
    assert "$tuned_params" not in result.default_boot_entry["args"]
    assert "$tuned_params" not in result.default_boot_entry["cmdline"]
    assert result.default_boot_entry["cmdline"].endswith(" audit_backlog_limit=8192")
    assert len(result.boot_entries) == 2
    assert not result.is_efi



"""
System block files under ``/sys/block``
=======================================

This module contains the following parsers:

StableWrites - file ``/sys/block/*/queue/stable_writes``
--------------------------------------------------------

MaxSegmentSize - file ``/sys/block/*/queue/max_segment_size``
-------------------------------------------------------------
"""
from insights import Parser, parser
from insights.core.exceptions import ParseException
from insights.specs import Specs


@parser(Specs.sys_block_queue_stable_writes)
class StableWrites(Parser):
    """
    Class for parsing the ``/sys/block/*/queue/stable_writes`` files.

    Typical content of the file is::

        1

    Examples:
        >>> type(block_stable_writes)
        <class 'insights.parsers.sys_block.StableWrites'>
        >>> block_stable_writes.stable_writes
        1
        >>> block_stable_writes.device
        'sda'

    Raises:
        ParseException: When content is empty or unparsable
    """

    def parse_content(self, content):
        if len(content) != 1:
            raise ParseException('Error: {0}'.format(content if content else 'empty file'))
        try:
            self._stable_writes = int(content[0].strip())
            self._device = self.file_path.split('/')[-3]
        except ValueError:
            raise ParseException("Error: unparsable content: {0}".format(content[0]))

    @property
    def stable_writes(self):
        """ int: Value of ``stable_writes``."""
        return self._stable_writes

    @property
    def device(self):
        """ str: Block device name from file path."""
        return self._device


@parser(Specs.sys_block_queue_max_segment_size)
class MaxSegmentSize(Parser):
    """
    Class for parsing the ``/sys/block/*/queue/max_segment_size`` files.

    Typical content of the file is::

        4294967295

    Examples:
        >>> type(max_segment_size)
        <class 'insights.parsers.sys_block.MaxSegmentSize'>
        >>> max_segment_size.max_segment_size
        4294967295
        >>> max_segment_size.device
        'sda'

    Raises:
        ParseException: When content is empty or unparsable
    """

    def parse_content(self, content):
        if len(content) != 1:
            raise ParseException('Error: {0}'.format(content or 'empty file'))
        try:
            self._max_segment_size = int(content[0].strip())
            self._device = self.file_path.split('/')[-3]
        except ValueError:
            raise ParseException("Error: unparsable content: {0}".format(content[0]))

    max_segment_size = property(lambda self: self._max_segment_size)
    device = property(lambda self: self._device)



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import bond_dynamic_lb
from insights.parsers.bond_dynamic_lb import BondDynamicLB
from insights.tests import context_wrap


CONTEXT_PATH = "/sys/class/net/bond0/bonding/tlb_dynamic_lb"
CONTEXT_PATH_1 = "/sys/class/net/bond1/bonding/tlb_dynamic_lb"
CONTEXT_PATH_2 = "/sys/class/net/bond2/bonding/tlb_dynamic_lb"
CONTEXT_PATH_3 = "/sys/class/net/test3/bonding/tlb_dynamic_lb"

BOND_LD_BALANCE = """
0
""".strip()

BOND_LD_BALANCE_1 = """
1
""".strip()

BOND_LD_BALANCE_2 = """
off
""".strip()

BOND_LD_BALANCE_NO = """

""".strip()


def test_netstat_doc_examples():
    env = {
        'tlb_bond': BondDynamicLB(context_wrap(BOND_LD_BALANCE_1, CONTEXT_PATH)),
    }
    failed, total = doctest.testmod(bond_dynamic_lb, globs=env)
    assert failed == 0


def test_bond_dynamic_lb_class():

    tlb_bond = BondDynamicLB(context_wrap(BOND_LD_BALANCE, CONTEXT_PATH))
    assert tlb_bond.bond_name == 'bond0'
    assert tlb_bond.dynamic_lb_status == 0

    tlb_bond = BondDynamicLB(context_wrap(BOND_LD_BALANCE_1, CONTEXT_PATH_1))
    assert tlb_bond.bond_name == 'bond1'
    assert tlb_bond.dynamic_lb_status == 1

    tlb_bond = BondDynamicLB(context_wrap(BOND_LD_BALANCE_1, CONTEXT_PATH_3))
    assert tlb_bond.bond_name == 'test3'
    assert tlb_bond.dynamic_lb_status == 1

    with pytest.raises(ParseException) as exc:
        bond_obj = BondDynamicLB(context_wrap(BOND_LD_BALANCE_2, CONTEXT_PATH))
        assert not bond_obj.bond_name
    assert 'Unrecognised Values' in str(exc)

    with pytest.raises(SkipComponent) as exc:
        bond_obj = BondDynamicLB(context_wrap(BOND_LD_BALANCE_NO, CONTEXT_PATH))
        assert not bond_obj.bond_name
    assert 'No Contents' in str(exc)



import pytest

from insights.core import JSONParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.tests import context_wrap


class MyJsonParser(JSONParser):
    pass


json_test_strings = {
    '{"a": "1", "b": "2"}': {'a': '1', 'b': '2'},
    '[{"a": "1", "b": "2"},{"a": "3", "b": "4"}]':
        [{'a': '1', 'b': '2'}, {'a': '3', 'b': '4'}]
}

JSON_CONTENT_WITH_EXTRA_HEADER_LINES_1 = """
INFO: The fwupdagent command is deprecated, use `fwupdmgr --json` instead
WARNING: UEFI firmware can not be updated in legacy BIOS mode
  See https://github.com/fwupd/fwupd/wiki/PluginFlag:legacy-bios for more information.
{"a": [{"b": "x"}, {"b": "y"}]}
""".strip()
JSON_CONTENT_WITH_EXTRA_HEADER_LINES_2 = """
INFO: The fwupdagent command is deprecated, use `fwupdmgr --json` instead
WARNING: UEFI firmware can not be updated in legacy BIOS mode
  See https://github.com/fwupd/fwupd/wiki/PluginFlag:legacy-bios for more information.
{}
""".strip()
JSON_CONTENT_WITH_EXTRA_HEADER_LINES_3 = """
INFO: Some info line from a command spec output
WARNING: Some warning line-1 from a command spec output
  Some warning line-2 from a command spec output.
""".strip()


def test_json_parser_success():
    for jsonstr in json_test_strings:
        ctx = context_wrap(jsonstr)
        my_json_parser = MyJsonParser(ctx)
        assert my_json_parser.data == json_test_strings[jsonstr]
        assert my_json_parser.unparsed_lines == []


def test_json_parser_failure():
    ctx = context_wrap("boom")
    with pytest.raises(ParseException) as ex:
        MyJsonParser(ctx)

    assert "MyJsonParser" in ex.value.args[0]


def test_json_parser_null_value():
    ctx = context_wrap("null")
    with pytest.raises(SkipComponent) as ex:
        MyJsonParser(ctx)

    assert "Empty input" == ex.value.args[0]


def test_json_parser_with_extra_header_lines():
    ctx = context_wrap(JSON_CONTENT_WITH_EXTRA_HEADER_LINES_1)
    my_json_parser = MyJsonParser(ctx)
    assert my_json_parser.data == {'a': [{'b': 'x'}, {'b': 'y'}]}
    assert my_json_parser.unparsed_lines == [
        "INFO: The fwupdagent command is deprecated, use `fwupdmgr --json` instead",
        "WARNING: UEFI firmware can not be updated in legacy BIOS mode",
        "  See https://github.com/fwupd/fwupd/wiki/PluginFlag:legacy-bios for more information."]

    ctx = context_wrap(JSON_CONTENT_WITH_EXTRA_HEADER_LINES_2)
    my_json_parser = MyJsonParser(ctx)
    assert my_json_parser.data == {}
    assert len(my_json_parser.unparsed_lines) == 3

    ctx = context_wrap(JSON_CONTENT_WITH_EXTRA_HEADER_LINES_3)
    with pytest.raises(ParseException) as ex:
        MyJsonParser(ctx)

    assert "couldn't parse json" in ex.value.args[0]



"""
Red Hat Subscription Manager Release
====================================

Combiner provides the Red Hat Subscription Manager release information from
the parsers :class:`insights.parsers.rhsm_releasever.RhsmReleaseVer`
and :class:`insights.parsers.rhui_release.RHUIReleaseVer`.
"""
from insights.core.plugins import combiner
from insights.parsers.rhsm_releasever import RhsmReleaseVer
from insights.parsers.rhui_release import RHUIReleaseVer


@combiner([RhsmReleaseVer, RHUIReleaseVer])
class RhsmRelease(object):
    """
    Combiner for parsers RhsmReleaseVer and RHUIReleaseVer.

    Examples:
        >>> type(rhsm_release)
        <class 'insights.combiners.rhsm_release.RhsmRelease'>
        >>> rhsm_release.set == '7.6'
        True
        >>> rhsm_release.major
        7
        >>> rhsm_release.minor
        6
    """
    def __init__(self, rhsm_release, rhui_release):
        self.set = None
        """ str: Release version string returned from the parsers """

        self.major = None
        """ int: Major version of the release """

        self.minor = None
        """ int: Minor version of the release """

        if rhsm_release is not None and rhsm_release.set:
            self.set = rhsm_release.set
            self.major = rhsm_release.major
            self.minor = rhsm_release.minor

        elif rhui_release is not None and rhui_release.set:
            self.set = rhui_release.set
            self.major = rhui_release.major
            self.minor = rhui_release.minor



import doctest

import pytest

from insights.core.exceptions import ParseException
from insights.parsers import ipa_conf
from insights.tests import context_wrap

IPA_DEFAULT_CONF = """
[global]
host = client.ipa.test
basedn = dc=ipa,dc=test
realm = IPA.TEST
domain = ipa.test
xmlrpc_uri = https://server.ipa.test/ipa/xml
"""

IPA_DEFAULT_MINIMAL = """
[global]
realm = IPA.TEST
server = server.ipa.test
"""


def test_doc_examples():
    env = {
        "ipaconfig": ipa_conf.IPAConfig(context_wrap(IPA_DEFAULT_CONF)),
    }
    failed, total = doctest.testmod(ipa_conf, globs=env)
    assert failed == 0


def test_ipa_default_conf():
    cfg = ipa_conf.IPAConfig(context_wrap(IPA_DEFAULT_CONF))

    assert cfg.ipa_section == "global"
    assert cfg.get("global", "host") == "client.ipa.test"

    assert cfg.server == "server.ipa.test"
    assert cfg.domain == "ipa.test"
    assert cfg.realm == "IPA.TEST"
    assert cfg.basedn == "dc=ipa,dc=test"
    assert cfg.ldap_uri == "ldap://server.ipa.test"
    assert cfg.xmlrpc_uri == "https://server.ipa.test/ipa/xml"
    assert cfg.jsonrpc_uri == "https://server.ipa.test/ipa/json"


def test_ipa_default_minimal():
    cfg = ipa_conf.IPAConfig(context_wrap(IPA_DEFAULT_MINIMAL))

    assert cfg.server == "server.ipa.test"
    assert cfg.domain == "ipa.test"
    assert cfg.realm == "IPA.TEST"
    assert cfg.basedn == "dc=ipa,dc=test"
    assert cfg.ldap_uri == "ldap://server.ipa.test"
    assert cfg.xmlrpc_uri == "https://server.ipa.test/ipa/xml"
    assert cfg.jsonrpc_uri == "https://server.ipa.test/ipa/json"


def test_ipa_missing_section():
    with pytest.raises(ParseException):
        ipa_conf.IPAConfig(context_wrap("[invalid]"))


def test_ipa_missing_settings():
    with pytest.raises(ParseException):
        ipa_conf.IPAConfig(context_wrap("[global]"))



"""
OsslFilesConfig - Commands ``/usr/lib/dracut/ossl-files --config``
==================================================================
Classes to parse ``/usr/lib/dracut/ossl-files --config`` command information.

"""

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ossl_files)
class OsslFilesConfig(CommandParser):
    """
    Parses output of ``/usr/lib/dracut/ossl-files --config`` command.

    Attributes:
        error_lines (list): List of error lines.
        conf_path (str): Configuration file path of OpenSSL.

    Sample input::

        /etc/pki/tls/openssl.cnf

    Examples:
        >>> type(openssl_cnf)
        <class 'insights.parsers.ossl_files.OsslFilesConfig'>
        >>> openssl_cnf.conf_path
        '/etc/pki/tls/openssl.cnf'
    """

    def parse_content(self, content):
        self.conf_path = None
        self.error_lines = []

        if len(content) == 1:
            line = content[0].strip()
            if line and line.startswith('/'):
                self.conf_path = line
            else:
                self.error_lines.append(line)
        else:
            for line in content:
                self.error_lines.append(line.strip())

        if not self.conf_path and not self.error_lines:
            raise SkipComponent



"""
Parsers for the Corosync Cluster Engine configurations
======================================================


Parsers included in this module are:

CorosyncConf - file ``/etc/corosync/corosync.conf``
---------------------------------------------------
"""
import string
from insights.core import ConfigParser
from insights.specs import Specs
from insights import parser

from insights.parsr import (EOF, Forward, InSet, LeftCurly, Lift, LineEnd,
        Literal, RightCurly, Many, Number, OneLineComment, PosMarker,
        skip_none, String, QuotedString, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section


def parse_doc(f, ctx=None, line_end="\n"):
    def to_entry(name, rest):
        if isinstance(rest, list):
            return Section(name=name.value, children=rest, lineno=name.lineno, src=ctx)
        return Directive(name=name.value, attrs=[rest], lineno=name.lineno, src=ctx)

    Sep = InSet(":=")
    Stmt = Forward()
    Num = Number & (WSChar | LineEnd)
    NULL = Literal("none", value=None)
    Comment = (WS >> OneLineComment("#").map(lambda x: None))
    BeginBlock = (WS >> LeftCurly << WS)
    EndBlock = (WS >> RightCurly << WS)
    Bare = String(set(string.printable) - (set(string.whitespace) | set("#{}'\"")))
    Name = WS >> PosMarker(String(string.ascii_letters + "_" + string.digits)) << WS
    Value = WS >> (Num | NULL | QuotedString | Bare) << WS
    Block = BeginBlock >> Many(Stmt).map(skip_none) << EndBlock
    Stanza = (Lift(to_entry) * Name * (Block | (Sep >> Value))) | Comment
    Stmt <= WS >> Stanza << WS
    Doc = Many(Stmt).map(skip_none)
    Top = Doc + EOF

    return Entry(children=Top(f)[0], src=ctx)


@parser(Specs.corosync_conf)
class CorosyncConf(ConfigParser):
    """Parse the output of the file ``/etc/corosync/corosync.conf`` using
    the ``ConfigParser`` base class. It exposes the corosync
    configuration through the parsr query interface.

    The parameters in the directives are referred from the manpage of
    ``corosync.conf``. See ``man 8 corosync.conf`` for more info.

    Sample content of the file ``/etc/corosync/corosync.conf`` ::

        totem {
            version: 2
            secauth: off
            cluster_name: tripleo_cluster
            transport: udpu
            token: 10000
        }

        nodelist {
            node {
                ring0_addr: overcloud-controller-0
                nodeid: 1
            }

            node {
                ring0_addr: overcloud-controller-1
                nodeid: 2
            }

            node {
                ring0_addr: overcloud-controller-2
                nodeid: 3
            }
        }

        quorum {
            provider: corosync_votequorum
        }

        logging {
            to_logfile: yes
            logfile: /var/log/cluster/corosync.log
            to_syslog: yes
        }


    Example:

        >>> from insights.parsr.query import first, last
        >>> corosync_conf['quorum']['provider'][first].value
        'corosync_votequorum'
        >>> corosync_conf['totem']['token'][first].value
        10000
        >>> corosync_conf['nodelist']['node']['nodeid'][last].value
        3

    """
    def parse_doc(self, content):
        return parse_doc("\n".join(content), ctx=self)



r"""
Journald configuration
======================

Combiner for parsing of journald configuration. man journald.conf describes where various
journald config files can reside and how they take precedence one over another. The combiner
implements the logic and provides an interface for querying active settings.

The journald.conf file is a key=value file with hash comments.

The parsers this combiner uses process only active settings (lines that are not commented out). The
resulting settings (after being processed by the precedence evaluation algorithm) are then provided
by the `get_active_settings_value` method and `active_settings` dictionary and by the
`get_active_setting_value_and_file_name` method and `active_settings_with_file_name` dictionary.

Options that are commented out are not returned - a rule using this parser has to be aware of which
default value is assumed by systemd if the particular option is not specified.

Priority from lowest to highest:

* built-in defaults (the same as the default commented entries in /etc/systemd/journald.conf)
* /etc/systemd/journald.conf
* \*.conf in whatever directory in lexicographic order from lowest to highest
* if two \*.conf files with the same name are both in /usr/lib and /etc, the file in /etc wholly
  overwrites the file in /usr/lib

from man journald.conf in RHEL 7.3:

CONFIGURATION DIRECTORIES AND PRECEDENCE

       Default configuration is defined during compilation, so a configuration
       file is only needed when it is necessary to deviate from those
       defaults. By default the configuration file in /etc/systemd/ contains
       commented out entries showing the defaults as a guide to the
       administrator. This file can be edited to create local overrides.

       When packages need to customize the configuration, they can install
       configuration snippets in /usr/lib/systemd/\*.conf.d/. Files in /etc/
       are reserved for the local administrator, who may use this logic to
       override the configuration files installed by vendor packages. The main
       configuration file is read before any of the configuration directories,
       and has the lowest precedence; entries in a file in any configuration
       directory override entries in the single configuration file. Files in
       the \*.conf.d/ configuration subdirectories are sorted by their filename
       in lexicographic order, regardless of which of the subdirectories they
       reside in. If multiple files specify the same option, the entry in the
       file with the lexicographically latest name takes precedence. It is
       recommended to prefix all filenames in those subdirectories with a
       two-digit number and a dash, to simplify the ordering of the files.

       To disable a configuration file supplied by the vendor, the recommended
       way is to place a symlink to /dev/null in the configuration directory
       in /etc/, with the same filename as the vendor configuration file.

Examples:

    >>> conf = shared[JournaldConfAll]
    >>> conf.get_active_setting_value('Storage')
    'auto'
    >>> 'Storage' in conf.active_settings_with_file_name
    True
    >>> conf.get_active_setting_value_and_file_name('Storage')
    ('auto', '/etc/systemd/journald.conf')
"""

from insights.core.plugins import combiner
from insights.parsers.journald_conf import EtcJournaldConf, EtcJournaldConfD, UsrJournaldConfD


# TODO - further insights work - convert this to a generic option & file priority evaluator for
#        other combiners.

@combiner(EtcJournaldConf, optional=[EtcJournaldConfD, UsrJournaldConfD])
class JournaldConfAll(object):
    """
    Combiner for accessing files from the parsers EtcJournaldConf, EtcJournaldConfD, UsrJournaldConfD
    and evaluating effective active settings based on the rules of file priority and file shadowing
    as described in man journald.conf.

    Can be later refactored to a combiner for parsing all configuration files with key=option lines,
    like journald files.

    Rules of evaluation:

    * Files from EtcJournaldConfD wholly shadow/overwrite files from UsrJournaldConfD with identical
      names.
    * Files ordered by name from lowest priority to highest (a.conf has lower priority than b.conf).
    * Option values overwritten by the file with the highest priority.
    * The one central file has either the lowest priority or the highest priority, based on the
      central_file_lowest_prio argument.

    That is:

    * An entire file in UsrJournaldConfD is overwritten by a same-named file from EtcJournaldConfD.
    * A single option value is overwritten when another file with a higher priority has an option
      with the same option name.

    Example of file precedence::

        /etc/systemd/journald.conf:
            key0=value0
            key1=value1

        /usr/lib/systemd/journald.conf.d/a.conf:
            key2=value2
            key3=value3
            key4=value4
            key1=value5

        /usr/lib/systemd/journald.conf.d/b.conf:
            key5=value6
            key6=value7
            key1=value8
            key2=value9
            key4=value10

        /usr/lib/systemd/journald.conf.d/c.conf:
            key7=value11
            key5=value12
            key1=value13

        /etc/systemd/journald.conf.d/b.conf:
            key1=value14
            key5=value15

        the resulting configuration:
            key0=value0
            key1=value13 # c.conf has highest priority
            key2=value2 # b.conf from /usr is shadowed by b.conf from /etc so value from a.conf is used
            key3=value3
            key4=value4 # b.conf from /usr is shadowed by b.conf from /etc so value from a.conf is used
            key5=value12 # c.conf has higher priority than b.conf
            # key6 doesn't exist because b.conf from /usr is shadowed by b.conf from /etc
            key7=value11
    """
    def __init__(self, journal_conf, journal_conf_d, usr_journal_conf_d):

        # preparation for future possible refactoring into a more general combiner
        central_file_lowest_prio = True

        # comments in this method describe journald configuration; it should work for similar ones
        etc_confd = {}  # parser instances indexed by file name
        usr_confd = {}  # parser instances indexed by file name
        if journal_conf_d:
            for parser_instance in journal_conf_d:
                etc_confd[parser_instance.file_name] = parser_instance
        if usr_journal_conf_d:
            for parser_instance in usr_journal_conf_d:
                usr_confd[parser_instance.file_name] = parser_instance

        files_shadowed_not_used = set()  # full file paths of files that are shadowed by others
        effective_confd = {}  # deduplicated *.conf files, taking shadowing /usr by /etc into account
        for file_name, parser_instance in usr_confd.items():
            effective_confd[file_name] = parser_instance
        # /etc/systemd/journald.conf.d/*.conf shadow /usr/lib/systemd/journald.conf.d/*.conf files
        #  with the same name. The following loop overwrites these same-named files by their /etc
        #  counterparts:
        for file_name, parser_instance in etc_confd.items():
            if file_name in effective_confd:
                shadowed_file_name = effective_confd[file_name].file_path
                if shadowed_file_name:
                    # empty and None file names are not added (that is invalid anyway)
                    files_shadowed_not_used.add(shadowed_file_name)
            effective_confd[file_name] = parser_instance

        files_shadowed_not_used = sorted(files_shadowed_not_used)  # deterministic behavior, sorted paths
        sorted_file_names = sorted(effective_confd.keys(), key=str)

        parsers_list = [effective_confd[file_name] for file_name in sorted_file_names]
        if central_file_lowest_prio:
            parsers_list = [journal_conf] + parsers_list
        else:
            parsers_list = parsers_list + [journal_conf]

        files_used_priority_order = []  # from lowest to highest priority, not including empty files
        # storing only the active values as (val, file_name), taking precedence rules into account
        resulting_options_with_file_name = {}
        # *.conf files from the lowest priority to the highest, so that the last value stays
        for parser_instance in parsers_list:
            if parser_instance.active_settings:  # do not iterate if empty or None
                if parser_instance.file_path:
                    # empty and None file names are not added (that is invalid anyway), see test_11
                    files_used_priority_order.append(parser_instance.file_path)
                for k, v in parser_instance.active_settings.items():
                    resulting_options_with_file_name[k] = (v, parser_instance.file_path)

        # not named simply as `active_settings` so as not to confuse the contents with the
        #  `active_settings` dictionary in parsers/journald_conf.py
        #   (dict[str, str] vs. dict[str, tuple[str, str]])
        self.active_settings_with_file_name = resulting_options_with_file_name

        # Not saving directly to self so that if this function fails in the middle, nothing is saved
        #  for the offshoot chance that an exception would be swallowed and the invalid instance
        #  used - prevents incomplete data from being used.
        self.files_shadowed_not_used = files_shadowed_not_used
        self.files_used_priority_order = files_used_priority_order
        super(JournaldConfAll, self).__init__()

    def get_active_setting_value(self, setting_name):
        """
        Access active setting value by setting name.

        Args:
            setting_name (string): Setting name
        """
        return self.active_settings_with_file_name[setting_name][0]

    def get_active_setting_value_and_file_name(self, setting_name):
        """
        Access active setting value by setting name. Returns the active setting value and file name
        of the file in which it is defined. Other files that also specify the setting but are
        shadowed are ignored and not reported.

        Args:
            setting_name (string): Setting name

        Returns:
            tuple[str, str]: setting value, file name
        """
        return self.active_settings_with_file_name[setting_name]



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import ksmstate
from insights.parsers.ksmstate import KSMState
from insights.tests import context_wrap

KSMSTATE0 = "0"
KSMSTATE1 = "1"
KSMSTATE2 = "2"
KSMSTATE_ab0 = ""
KSMSTATE_ab1 = "abc"
KSMSTATE_ab2 = """
abc
1
"""


def test_ksmstate():
    ksm = KSMState(context_wrap(KSMSTATE0))
    assert ksm.is_running is False
    assert ksm.value == '0'

    ksm = KSMState(context_wrap(KSMSTATE1))
    assert ksm.is_running is True
    assert ksm.value == '1'

    ksm = KSMState(context_wrap(KSMSTATE2))
    assert ksm.is_running is False
    assert ksm.value == '2'


def test_ksmstate_exp():
    with pytest.raises(SkipComponent):
        KSMState(context_wrap(KSMSTATE_ab0))

    with pytest.raises(ParseException):
        KSMState(context_wrap(KSMSTATE_ab1))

    with pytest.raises(ParseException):
        KSMState(context_wrap(KSMSTATE_ab2))


def test_ksmstate_doc():
    env = {'ksm': KSMState(context_wrap(KSMSTATE0))}
    failed, total = doctest.testmod(ksmstate, globs=env)
    assert failed == 0



from insights.core.ls_parser import FilePermissions as FP
from insights.util import deprecated


class FilePermissions(FP):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.core.ls_parser.FilePermissions` instead.
    """

    def __init__(self, *args, **kwargs):
        deprecated(
            FilePermissions, "Please use insights.core.ls_parser.FilePermissions instead.", "3.7.0"
        )
        super(FilePermissions, self).__init__(*args, **kwargs)



"""
nmap - command ``nmap``
=======================

Parser contains in this module is:

NmapSsh - command ``/usr/bin/nmap --script ssh2-enum-algos -sV -p 22 127.0.0.1``
--------------------------------------------------------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.nmap_ssh)
class NmapSsh(CommandParser, dict):
    """
    `/usr/bin/nmap --script ssh2-enum-algos -sV -p 22 127.0.0.1` prints the algorithms configuration of ssh

    Typical content looks like::

        Starting Nmap 7.92 ( https://nmap.org ) at 2024-05-03 01:52 UTC
        Nmap scan report for localhost (127.0.0.1)
        Host is up (0.00024s latency).

        PORT   STATE SERVICE VERSION
        22/tcp open  ssh     OpenSSH 8.0 (protocol 2.0)
        | ssh2-enum-algos:
        |   kex_algorithms: (4)
        |       curve25519-sha256
        |       curve25519-sha256@libssh.org
        |       ecdh-sha2-nistp256
        |       ecdh-sha2-nistp384
        |   server_host_key_algorithms: (1)
        |       rsa-sha2-512
        |   encryption_algorithms: (1)
        |       aes256-gcm@openssh.com
        |   mac_algorithms: (2)
        |       hmac-sha2-256-etm@openssh.com
        |       hmac-sha1-etm@openssh.com
        |   compression_algorithms: (2)
        |       none
        |_      zlib@openssh.com

        Service detection performed. Please report any incorrect results at https://nmap.org/submit/ .
        Nmap done: 1 IP address (1 host up) scanned in 1.03 seconds

    Examples:
        >>> type(nmap_ssh)
        <class 'insights.parsers.nmap.NmapSsh'>
        >>> nmap_ssh['server_host_key_algorithms']
        ['rsa-sha2-512']

    Raises:
        insights.core.exceptions.SkipComponent: if the output of the ``repquota -agnpuv`` command is empty or the format is not expected.
    """

    _ALGO_SPLITERS = tuple(["_algorithms: (", "_algorithms ("])

    def parse_content(self, content):
        data = {}
        current_key = ""

        for line in content:
            line_strip = line.strip()
            if not line_strip:
                continue

            if line_strip.startswith("|"):
                if self._ALGO_SPLITERS[0] in line_strip:
                    current_key = line_strip.split(":")[0][1:].strip()
                    data[current_key] = []
                elif self._ALGO_SPLITERS[1] in line_strip:
                    current_key = line_strip.split()[1]
                    data[current_key] = []
                elif not line_strip.endswith(':'):
                    data[current_key].append(line.split()[-1])

        if not data:
            raise SkipComponent("The output format is not expected")

        self.update(data)



import doctest
from insights.parsers import eap_json_reports
from insights.parsers.eap_json_reports import EAPJSONReports
from insights.tests import context_wrap

EAP_JSON_DATA_1 = """
{
    "version": "1.0.0",
    "idHash": "e38277334d0f6b6fdc6f3b831fb102cdd70f04faab5c38b0be36fb1aacb4236eca86643d3e88fac6f47fabda8f9990e80436afa5a0e071890fd572238e727d2e",
    "basic": {
        "app.user.dir": "/opt/t_eap/jboss-eap-7.4",
        "java.specification.version": "1.8",
        "java.runtime.version": "1.8.0_362-b09",
        "jvm.args": [
            "-D[Standalone]",
            "-verbose:gc"
        ],
        "java.vm.name": "OpenJDK 64-Bit Server VM",
        "java.vm.specification.version": "1.8",
        "system.cores.logical": 2
    },
    "jars": {
        "version": "1.0.0",
        "jars": [
            {
                "name": "jboss-modules.jar",
                "version": "1.12.0.Final-redhat-00001",
                "attributes": {
                    "sha1Checksum": "82a8c99551533f4448675f273665cb6d7b750511",
                    "version": "1.12.0.Final-redhat-00001"
                }
            }
        ]
    },
    "eap": {
        "name": "Red Hat JBoss EAP",
        "version": "1.0.0",
        "eap-modules": {
            "version": "1.0.0",
            "jars": [
                {
                    "name": "activemq-artemis-native-1.0.2.redhat-00004.jar",
                    "version": "1.0.2.redhat-00004",
                    "attributes": {
                        "sha1Checksum": "1aaf77fa3aed8377c375770529fa66d5faa5c9c1",
                        "overridden": "unknown",
                        "version": "1.0.2.redhat-00004"
                    }
                },
                {
                    "name": "yasson-1.0.10.redhat-00001.jar",
                    "version": "1.0.10.redhat-00001",
                    "attributes": {
                        "sha1Checksum": "75b7dedb6d33b1ed26598a6cc57b75491adba959",
                        "Implementation-Vendor": "Oracle Corporation",
                        "path": "system/layers/base/org/eclipse/yasson/main/yasson-1.0.10.redhat-00001.jar",
                        "groupId": "org.eclipse",
                        "sha256Checksum": "88609252b5a89c11618a78365c04c8335ce7be8cc33629a9f53ee42eff7f4567",
                        "artifactId": "yasson",
                        "sha512Checksum": "1afdbc109d2e5738f5c9f9fcf1e2090214d2beb50f48f2b1ed084f7fa895aeb63035452eb0242a7a234aa38799c9ab3166381cd2a799fb6e2273823fa30861bf",
                        "Implementation-Vendor-Id": "org.eclipse",
                        "overridden": "unknown",
                        "version": "1.0.10.redhat-00001"
                    }
                }
            ]
        },
        "eap-configuration": {
            "version": "16.0.0",
            "configuration": {
                "launch-type": "STANDALONE",
                "server-state": "running",
                "suspend-state": "RUNNING",
                "uuid": "254d541c-a9ab-4df4-8bb7-2bc84447beb7",
                "core-service": {
                    "management": {
                        "access": {
                            "authorization": {
                                "provider": null
                            }
                        },
                        "ldap-connection": null,
                        "management-interface": {
                            "http-interface": {
                                "allowed-origins": null,
                                "console-enabled": true,
                                "constant-headers": null,
                                "http-authentication-factory": null,
                                "http-upgrade": null,
                                "http-upgrade-enabled": true,
                                "sasl-protocol": "remote",
                                "secure-socket-binding": null,
                                "security-realm": null,
                                "server-name": null,
                                "socket-binding": "management-http",
                                "ssl-context": null
                            }
                        },
                        "security-realm": null,
                        "service": {
                            "management-operations": {
                                "active-operation": {
                                    "-1638739660": {
                                        "access-mechanism": null,
                                        "address": [],
                                        "caller-thread": "ServerService Thread Pool -- 1",
                                        "cancelled": false,
                                        "domain-rollout": false,
                                        "domain-uuid": null,
                                        "exclusive-running-time": -1,
                                        "execution-status": "executing",
                                        "operation": "read-resource",
                                        "running-time": 141095723
                                    }
                                }
                            }
                        }
                    },
                    "service-container": {},
                    "module-loading": {
                        "module-roots": [
                            "/opt/t_eap/jboss-eap-7.4/modules",
                            "/opt/t_eap/jboss-eap-7.4/modules/system/layers/base"
                        ]
                    },
                    "server-environment": {
                        "base-dir": "/opt/t_eap/jboss-eap-7.4/standalone",
                        "config-dir": "/opt/t_eap/jboss-eap-7.4/standalone/configuration",
                        "config-file": "/opt/t_eap/jboss-eap-7.4/standalone/configuration/standalone.xml",
                        "content-dir": "/opt/t_eap/jboss-eap-7.4/standalone/data/content",
                        "data-dir": "/opt/t_eap/jboss-eap-7.4/standalone/data",
                        "deploy-dir": "/opt/t_eap/jboss-eap-7.4/standalone/data/content",
                        "ext-dirs": [
                            "/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.362.b09-2.el8_7.x86_64/jre/lib/ext",
                            "/usr/java/packages/lib/ext"
                        ],
                        "home-dir": "/opt/t_eap/jboss-eap-7.4",
                        "host-name": "rhel8testing",
                        "initial-running-mode": "NORMAL",
                        "launch-type": "STANDALONE",
                        "log-dir": "/opt/t_eap/jboss-eap-7.4/standalone/log",
                        "modules-dir": "/opt/t_eap/jboss-eap-7.4/modules",
                        "node-name": "rhel8testing",
                        "qualified-host-name": "rhel8testing",
                        "server-name": "rhel8testing",
                        "start-gracefully": true,
                        "start-suspended": false,
                        "temp-dir": "/opt/t_eap/jboss-eap-7.4/standalone/tmp"
                    },
                    "patching": {
                        "cumulative-patch-id": "base",
                        "patches": [],
                        "version": "7.4.11.GA",
                        "addon": null,
                        "layer": {
                            "base": {
                                "cumulative-patch-id": "base",
                                "patches": []
                            }
                        },
                        "patch-stream": {
                            "JBoss EAP": {
                                "cumulative-patch-id": "base",
                                "patches": [],
                                "version": "7.4.11.GA",
                                "addon": null,
                                "layer": {
                                    "base": {
                                        "cumulative-patch-id": "base",
                                        "patches": []
                                    }
                                }
                            }
                        }
                    }
                },
                "deployment": {
                    "helloworld.war": {
                        "content": [
                            {
                                "hash": {
                                    "BYTES_VALUE": "y/AofkRXxfZVXaEY/HeBXBQ+LSo="
                                }
                            }
                        ],
                        "disabled-time": null,
                        "disabled-timestamp": null,
                        "enabled": true,
                        "enabled-time": 1686660031851,
                        "enabled-timestamp": "2023-06-13 12:40:31,851 UTC",
                        "managed": true,
                        "name": "helloworld.war",
                        "owner": [
                            {
                                "subsystem": "deployment-scanner"
                            },
                            {
                                "scanner": "default"
                            }
                        ],
                        "persistent": false,
                        "runtime-name": "helloworld.war",
                        "status": "OK",
                        "subdeployment": null,
                        "subsystem": {
                            "undertow": {
                                "active-sessions": 0,
                                "servlet": null,
                                "websocket": null
                            },
                            "logging": {
                                "configuration": {
                                    "default": {
                                        "error-manager": null,
                                        "logger": {
                                            "ROOT": {
                                                "filter": null,
                                                "handlers": [
                                                    "FILE",
                                                    "CONSOLE"
                                                ],
                                                "level": "INFO",
                                                "use-parent-handlers": null
                                            },
                                            "com.arjuna": {
                                                "filter": null,
                                                "handlers": [],
                                                "level": "WARN",
                                                "use-parent-handlers": true
                                            }
                                        },
                                        "pojo": null
                                    }
                                }
                            }
                        }
                    }
                },
                "deployment-overlay": null,
                "extension": {
                    "org.jboss.as.clustering.infinispan": {
                        "module": "org.jboss.as.clustering.infinispan",
                        "subsystem": {
                            "infinispan": {
                                "management-major-version": 14,
                                "management-micro-version": 0,
                                "management-minor-version": 0
                            }
                        }
                    },
                    "org.wildfly.extension.undertow": {
                        "module": "org.wildfly.extension.undertow",
                        "subsystem": {
                            "undertow": {
                                "management-major-version": 11,
                                "management-micro-version": 0,
                                "management-minor-version": 0
                            }
                        }
                    }
                },
                "interface": {
                    "management": {
                        "any": null,
                        "any-address": null,
                        "inet-address": {
                            "EXPRESSION_VALUE": "${jboss.bind.address.management:127.0.0.1}"
                        },
                        "link-local-address": null,
                        "up": null,
                        "virtual": null
                    },
                    "public": {
                        "any": null,
                        "any-address": null,
                        "inet-address": {
                            "EXPRESSION_VALUE": "${jboss.bind.address:127.0.0.1}"
                        },
                        "link-local-address": null,
                        "up": null,
                        "virtual": null
                    }
                },
                "path": {
                    "jboss.server.log.dir": {
                        "name": "jboss.server.log.dir",
                        "path": "/opt/t_eap/jboss-eap-7.4/standalone/log",
                        "read-only": true,
                        "relative-to": null
                    },
                    "java.home": {
                        "name": "java.home",
                        "path": "/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.362.b09-2.el8_7.x86_64/jre",
                        "read-only": true,
                        "relative-to": null
                    }
                },
                "socket-binding-group": {
                    "standard-sockets": {
                        "default-interface": "public",
                        "name": "standard-sockets",
                        "port-offset": {
                            "EXPRESSION_VALUE": "${jboss.socket.binding.port-offset:0}"
                        },
                        "local-destination-outbound-socket-binding": null,
                        "remote-destination-outbound-socket-binding": {
                            "mail-smtp": {
                                "fixed-source-port": false,
                                "host": {
                                    "EXPRESSION_VALUE": "${jboss.mail.server.host:localhost}"
                                },
                                "port": {
                                    "EXPRESSION_VALUE": "${jboss.mail.server.port:25}"
                                },
                                "source-interface": null,
                                "source-port": null
                            }
                        },
                        "socket-binding": {
                            "ajp": {
                                "bound": false,
                                "bound-address": null,
                                "bound-port": null,
                                "client-mappings": null,
                                "fixed-port": false,
                                "interface": null,
                                "multicast-address": null,
                                "multicast-port": null,
                                "name": "ajp",
                                "port": {
                                    "EXPRESSION_VALUE": "${jboss.ajp.port:8009}"
                                }
                            },
                            "txn-status-manager": {
                                "bound": false,
                                "bound-address": null,
                                "bound-port": null,
                                "client-mappings": null,
                                "fixed-port": false,
                                "interface": null,
                                "multicast-address": null,
                                "multicast-port": null,
                                "name": "txn-status-manager",
                                "port": 4713
                            }
                        }
                    }
                },
                "subsystem": {
                    "core-management": {
                        "process-state-listener": null,
                        "service": null
                    },


                    "jca": {
                        "archive-validation": {
                            "archive-validation": {
                                "enabled": true,
                                "fail-on-error": true,
                                "fail-on-warn": false
                            }
                        },
                        "workmanager": {
                            "default": {
                                "elytron-enabled": false,
                                "name": "default",
                                "long-running-threads": {
                                    "default": {
                                        "allow-core-timeout": false,
                                        "keepalive-time": {
                                            "time": 10,
                                            "unit": "SECONDS"
                                        },
                                        "largest-thread-count": 0,
                                        "thread-factory": null
                                    }
                                },
                                "short-running-threads": {
                                    "default": {
                                        "allow-core-timeout": false,
                                        "thread-factory": null
                                    }
                                },
                                "statistics": {
                                    "local": {
                                        "dowork-accepted": 0,
                                        "workmanager-statistics-enabled": true
                                    }
                                }
                            }
                        }
                    },
                    "jdr": {},
                    "jmx": {
                        "non-core-mbean-sensitivity": null,
                        "remoting-connector": {
                            "jmx": {
                                "use-management-endpoint": true
                            }
                        }
                    },
                    "jpa": {
                        "default-datasource": null,
                        "default-extended-persistence-inheritance": "DEEP"
                    },
                    "jsf": {
                        "default-jsf-impl-slot": "main",
                        "disallow-doctype-decl": null
                    },
                    "mail": {
                        "mail-session": {
                            "default": {
                                "debug": false,
                                "from": null,
                                "jndi-name": "java:jboss/mail/Default",
                                "custom": null,
                                "server": {
                                    "smtp": {
                                        "credential-reference": null,
                                        "outbound-socket-binding-ref": "mail-smtp",
                                        "password": null,
                                        "ssl": false,
                                        "tls": false,
                                        "username": null
                                    }
                                }
                            }
                        }
                    },

                    "pojo": {},
                    "security": {
                        "deep-copy-subject-mode": null,
                        "vault": null
                    },
                    "security-manager": {
                        "deployment-permissions": {
                            "default": {
                                "maximum-permissions": [
                                    {
                                        "class": "java.security.AllPermission"
                                    }
                                ],
                                "minimum-permissions": null
                            }
                        }
                    },
                    "transactions": {
                        "average-commit-time": 0,
                        "commit-markable-resource": null,
                        "log-store": {
                            "log-store": {
                                "expose-all-logs": false,
                                "type": "default",
                                "transactions": null
                            }
                        }
                    },
                    "undertow": {
                        "default-security-domain": null,
                        "buffer-cache": {
                            "default": {
                                "buffer-size": 1024,
                                "buffers-per-region": 1024,
                                "max-regions": 10
                            }
                        },
                        "byte-buffer-pool": null,
                        "configuration": {
                            "filter": {
                                "custom-filter": null,
                                "rewrite": null
                            },
                            "handler": {
                                "file": {
                                    "welcome-content": {
                                        "cache-buffer-size": 1024,
                                        "path": {
                                            "EXPRESSION_VALUE": "${jboss.home.dir}/welcome-content"
                                        },
                                        "safe-symlink-paths": null
                                    }
                                },
                                "reverse-proxy": null
                            }
                        },
                        "server": {
                            "default-server": {
                                "default-host": "default-host",
                                "servlet-container": "default",
                                "ajp-listener": null,
                                "host": {
                                    "default-host": {
                                        "alias": [
                                            "localhost"
                                        ],
                                        "default-response-code": 404,
                                        "setting": {
                                            "http-invoker": {
                                                "http-authentication-factory": null,
                                                "path": "wildfly-services",
                                                "security-realm": null
                                            }
                                        }
                                    }
                                },
                                "http-listener": {
                                    "default": {
                                        "allow-encoded-slash": false,
                                        "bytes-sent": 0,
                                        "certificate-forwarding": false,
                                        "decode-url": true,
                                        "disallowed-methods": [
                                            "TRACE"
                                        ],
                                        "enable-http2": true,
                                        "worker": "default",
                                        "write-timeout": null
                                    }
                                },
                                "https-listener": {
                                    "https": {
                                        "allow-encoded-slash": false,
                                        "write-timeout": null
                                    }
                                }
                            }
                        },
                        "servlet-container": {
                            "default": {
                                "allow-non-standard-wrappers": false,
                                "mime-mapping": null,
                                "setting": {
                                    "jsp": {
                                        "check-interval": 0,
                                        "development": false,
                                        "x-powered-by": true
                                    },
                                    "websockets": {
                                        "buffer-pool": "default",
                                        "deflater-level": 0,
                                        "dispatch-to-worker": true,
                                        "per-message-deflate": false,
                                        "worker": "default"
                                    }
                                },
                                "welcome-file": null
                            }
                        }
                    },
                    "webservices": {
                        "modify-wsdl-address": true,
                        "endpoint-config": {
                            "Standard-Endpoint-Config": {
                                "post-handler-chain": null,
                                "pre-handler-chain": null,
                                "property": null
                            },
                            "Recording-Endpoint-Config": {
                                "post-handler-chain": null,
                                "pre-handler-chain": {
                                    "recording-handlers": {
                                        "protocol-bindings": "##SOAP11_HTTP ##SOAP11_HTTP_MTOM ##SOAP12_HTTP ##SOAP12_HTTP_MTOM",
                                        "handler": {
                                            "RecordingHandler": {
                                                "class": "org.jboss.ws.common.invocation.RecordingServerHandler"
                                            }
                                        }
                                    }
                                },
                                "property": null
                            }
                        }
                    }
                }
            }
        },
        "eap-deployments": {
            "version": "1.0.0",
            "deployments": [
                {
                    "name": "helloworld.war",
                    "archives": [
                        {
                            "name": "helloworld.war",
                            "version": " ",
                            "attributes": {
                                "sha1Checksum": "cbf0287e4457c5f6555da118fc77815c143e2d2a",
                                "sha256Checksum": "f41848d63c4098b96beb5c3eb51139de48757738fa093aca1bf666c95608a6ca",
                                "sha512Checksum": "70389a1e8d8581ec6706e4d731aa617d09864a602059b8bb505c68f311f83b9cc45b66f3abd25593dd573675b2df4e5fbbc1008a8f0f03fd5ae60e2f323bd85d",
                                "deployment": "helloworld.war"
                            }
                        }
                    ]
                }
            ]
        }
    }
}
""".strip()


def test_eap_json_report():
    result = EAPJSONReports(context_wrap(EAP_JSON_DATA_1))
    assert result.data['version'] == '1.0.0'
    assert result.data['basic']['app.user.dir'] == "/opt/t_eap/jboss-eap-7.4"


def test_doc_examples():
    env = {
        'eap_json_report': EAPJSONReports(context_wrap(EAP_JSON_DATA_1)),
    }
    failed, total = doctest.testmod(eap_json_reports, globs=env)
    assert failed == 0
